@@ -24,30 +24,45 @@ Commenter::Commenter(PumpController *pumpController, QWidget *parent) : QWidget(
 {
     this->pController = pumpController;
 
+
+    this->showAllCommentsLinkLabel = new QLabel("<a href=\"showall://\">"
+                                                + tr("Show All Comments")
+                                                + "</a>");
+    showAllCommentsLinkLabel->setContextMenuPolicy(Qt::NoContextMenu);
+    QFont showAllFont;
+    showAllFont.setPointSize(showAllFont.pointSize() - 3);
+    showAllCommentsLinkLabel->setFont(showAllFont);
+    connect(showAllCommentsLinkLabel, SIGNAL(linkActivated(QString)),
+            this, SLOT(requestAllComments()));
+
+
     this->commentsTextBrowser = new QTextBrowser();
     commentsTextBrowser->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
     commentsTextBrowser->setWordWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+    commentsTextBrowser->setOpenExternalLinks(true);
     QFont commentsFont;
-    commentsFont.setPointSize(commentsFont.pointSize()-1); // 1 point less than default
+    commentsFont.setPointSize(commentsFont.pointSize() - 1); // 1 point less than default
     commentsTextBrowser->setFont(commentsFont);
     commentsTextBrowser->setSizePolicy(QSizePolicy::Minimum,
-                                       QSizePolicy::Fixed);
-    commentsTextBrowser->hide(); // hide until setComments() is called, if there are any
+                                       QSizePolicy::Maximum);
+
+    // Hide these until setComments() is called, if there are any comments
+    showAllCommentsLinkLabel->hide();
+    commentsTextBrowser->hide();
 
 
-    this->showAllCommentsButton = new QPushButton(tr("All"));
-    showAllCommentsButton->setToolTip(tr("Show all the comments"));
-    connect(showAllCommentsButton, SIGNAL(clicked()),
-            this, SLOT(requestAllComments()));
 
 
     this->commentComposer = new Composer(false); // forPublisher = false
     this->commentComposer->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
     this->commentComposer->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    this->commentComposer->setSizePolicy(QSizePolicy::Minimum,
+                                         QSizePolicy::Minimum);
 
     connect(commentComposer, SIGNAL(editingFinished()),  this, SLOT(sendComment()));
     connect(commentComposer, SIGNAL(editingCancelled()), this, SLOT(setMinimumMode()));
 
+
     this->commentButton = new QPushButton(QIcon::fromTheme("mail-message-new"),
                                           tr("&Comment"));
     commentButton->setSizePolicy(QSizePolicy::Maximum,
@@ -60,11 +75,11 @@ Commenter::Commenter(PumpController *pumpController, QWidget *parent) : QWidget(
 
 
     mainLayout = new QGridLayout();
-    mainLayout->addWidget(commentsTextBrowser,   0, 0, 1, 2);
-    mainLayout->addWidget(showAllCommentsButton, 0, 1, 1, 1, Qt::AlignRight | Qt::AlignTop);
-    mainLayout->addWidget(commentComposer,       1, 0, 2, 1);
-    mainLayout->addWidget(commentButton,         1, 1, 1, 1);
-    mainLayout->addWidget(cancelButton,          2, 1, 1, 1);
+    mainLayout->addWidget(showAllCommentsLinkLabel, 0, 0, 1, 2, Qt::AlignRight | Qt::AlignTop);
+    mainLayout->addWidget(commentsTextBrowser,      1, 0, 1, 2);
+    mainLayout->addWidget(commentComposer,          2, 0, 2, 1);
+    mainLayout->addWidget(commentButton,            2, 1, 1, 1);
+    mainLayout->addWidget(cancelButton,             3, 1, 1, 1);
     this->setLayout(mainLayout);
 
 
@@ -99,16 +114,22 @@ void Commenter::setComments(QVariantList commentsList)
             QString commentAuthorID = commentMap.toMap().value("author").toMap().value("id").toString();
             commentAuthorID.remove(0, 5); // remove "acct:" at the beginning
             QString commentContent = commentMap.toMap().value("content").toString();
-            QString commentTimestamp = commentMap.toMap().value("published").toString();
-            commentTimestamp = Timestamp::fuzzyTime(commentTimestamp);
+
+            QString timestamp = commentMap.toMap().value("published").toString();
+            QString commentExactTime = Timestamp::localTimeDate(timestamp);
+            QString commentFuzzyTime = Timestamp::fuzzyTime(timestamp);
 
             commentsString = "<table>"
                              "<tr>"
-                             "<td><div title=\"" + commentAuthorID + "\">"
-                             "<img src=\"" + commentAuthorAvatarFile + "\" height=\"32\" align=\"left\"></img>"
-                             "</div></td>"
-                             "<td><b>" + commentAuthorName + "</b> "
-                             "- <i>" + commentTimestamp + "</i></td>"
+                             "<td><span title=\"" + commentAuthorID + "\">"
+                             "<img src=\"" + commentAuthorAvatarFile + "\" "
+                             "height=\"32\" align=\"left\"></img>"
+                             "</span></td>"
+                             "<td><span title=\"" + commentAuthorID + "\">"
+                             "<b>" + commentAuthorName + "</b></span> "
+                             "- <span title=\"" + commentExactTime + "\">"
+                             "<i>" + commentFuzzyTime + "</i>"
+                             "</span></td>"
                              "</tr>"
                              "<tr>"
                              "<td></td>"  // Empty cell below avatar
@@ -121,7 +142,11 @@ void Commenter::setComments(QVariantList commentsList)
         }
         this->commentsTextBrowser->setHtml(commentsString);
 
+        this->showAllCommentsLinkLabel->show();
         this->commentsTextBrowser->show();
+
+        // Move scrollbar to the bottom
+        this->commentsTextBrowser->verticalScrollBar()->triggerAction(QScrollBar::SliderToMaximum);
     }
 }
 
@@ -140,6 +165,9 @@ void Commenter::setMinimumMode()
 
     // Clear formatting options like bold or italic
     this->commentComposer->setCurrentCharFormat(QTextCharFormat());
+
+    // Unblock updates to timeline, since we're done commenting
+    this->pController->setUpdatesToTimelineBlocked(false);
 }
 
 
@@ -151,6 +179,9 @@ void Commenter::setFullMode()
     this->cancelButton->show();
 
     this->commentComposer->setFocus();
+
+    // Block updates to timeline while we're commenting...
+    this->pController->setUpdatesToTimelineBlocked(true);
 }
 
 
@@ -158,6 +189,9 @@ void Commenter::setFullMode()
 void Commenter::requestAllComments()
 {
     emit allCommentsRequested();
+
+    disconnect(pController, SIGNAL(commentPosted()),
+               this, SLOT(requestAllComments()));
 }
 
 
@@ -176,6 +210,9 @@ void Commenter::sendComment()
 
         // Done posting the comment, hide
         setMinimumMode();
+
+        connect(pController, SIGNAL(commentPosted()),
+                this, SLOT(requestAllComments()));
     }
     else
     {
