@@ -59,18 +59,27 @@ Composer::Composer(bool forPublisher)
                              tr("Underline"),
                              this,
                              SLOT(makeUnderline()));
+
         toolsMenu->addSeparator();
+
+        toolsMenu->addAction(QIcon::fromTheme("format-justify-fill"),
+                             tr("Preformatted block"),
+                             this,
+                             SLOT(makePreformatted()));
+
+        toolsMenu->addSeparator();
+
         toolsMenu->addAction(QIcon::fromTheme("insert-link"),
                              tr("Make a link"),
                              this,
                              SLOT(makeLink()));
-        /*
         toolsMenu->addAction(QIcon::fromTheme("insert-image"),
                              tr("Insert an image from a web site"),
                              this,
                              SLOT(insertImage()));
-        */
+
         toolsMenu->addSeparator();
+
         toolsMenu->addAction(QIcon::fromTheme("dialog-cancel"),
                              tr("Cancel Post"),
                              this,
@@ -260,6 +269,25 @@ void Composer::makeUnderline()
 
 
 
+/*
+ * Put selected text into a <pre> block
+ *
+ */
+void Composer::makePreformatted()
+{
+    QString selectedText = this->textCursor().selectedText();
+
+    if (!selectedText.isEmpty())
+    {
+        this->textCursor().removeSelectedText();
+        this->insertHtml("<pre>" + selectedText + "</pre>");
+    }
+
+    this->setFocus();
+}
+
+
+
 
 /*
  * Convert selected text into a link
@@ -294,6 +322,8 @@ void Composer::makeLink()
 
     if (!link.isEmpty())
     {
+        link = link.trimmed(); // Remove possible spaces before or after
+
         this->textCursor().removeSelectedText();
         this->insertHtml("<a href=\"" + link + "\">" + selectedText + "</a>");
     }
@@ -313,34 +343,19 @@ void Composer::makeLink()
  */
 void Composer::insertImage()
 {
-    QString selectedText = this->textCursor().selectedText();
+    // FIXME: should make sure there is no text selected
+
     QString imageURL;
-
-    if (selectedText.isEmpty())
-    {
-        imageURL = QInputDialog::getText(this,
-                                         tr("Insert and image from a URL"),
-                                         tr("Type or paste the image address here.\n"),
-                                         QLineEdit::Normal,
-                                         "http://");
-        selectedText = "image/url";
-    }
-    else
-    {
-        imageURL = QInputDialog::getText(this,
-                                         tr("Insert and image from a URL"),
-                                         tr("Type or paste the image address here.\n"
-                                            "The selected text (%1) will be used as description.").arg(selectedText),
-                                         QLineEdit::Normal,
-                                         "http://");
-    }
-
+    imageURL = QInputDialog::getText(this,
+                                     tr("Insert an image from a URL"),
+                                     tr("Type or paste the image address here.\n"),
+                                     QLineEdit::Normal,
+                                     "http://");
 
 
     if (!imageURL.isEmpty())
     {
-        this->textCursor().removeSelectedText();
-        this->insertHtml("<img alt=\"" + selectedText + "\" src=\"" + imageURL + "\"></img>");
+        this->insertHtml("<img src=\"" + imageURL + "\" />");
     }
     else
     {
@@ -359,10 +374,31 @@ void Composer::insertImage()
  */
 void Composer::cancelPost()
 {
-    this->erase();
+    int cancelConfirmed = 1;
+
+    if (this->document()->isEmpty())
+    {
+        cancelConfirmed = 0; // Cancelling doesn't need confirmation if it's empty
+    }
+    else
+    {
+        cancelConfirmed = QMessageBox::question(this,
+                                                tr("Cancel message?"),
+                                                tr("Are you sure you want to cancel this message?"),
+                                                tr("&Yes, cancel it"), tr("&No"));
+
+    }
+
+
+    if (cancelConfirmed == 0)
+    {
+        this->erase();
+
+        // emit signal to make Publisher go back to minimum mode
+        emit editingCancelled();
+
+        qDebug() << "Post cancelled";
+    }
 
-    // emit signal to make Publisher go back to minimum mode
-    emit editingCancelled();
 
-    qDebug() << "Post cancelled";
 }
