@@ -28,6 +28,7 @@
 #include <QPushButton>
 #include <QMenu>
 #include <QInputDialog>
+#include <QMessageBox>
 
 #include <QDebug>
 
@@ -52,6 +53,7 @@ public slots:
     void makeItalic();
     void makeBoldAndItalic();
     void makeUnderline();
+    void makePreformatted();
     void makeLink();
     void insertImage();
 
