@@ -103,7 +103,7 @@ ConfigDialog::ConfigDialog(int updateInterval, int postsPerPage, int tabsPositio
     showNotificationsCombobox->addItem(QIcon::fromTheme("preferences-desktop-notification"),
                                        tr("As system notifications"));
     showNotificationsCombobox->addItem(QIcon::fromTheme("view-conversation-balloon"),
-                                       tr("Using own balloon notifications"));
+                                       tr("Using own notifications"));
     showNotificationsCombobox->addItem(QIcon::fromTheme("user-busy"), // dialog-cancel
                                        tr("Don't show notifications"));
     showNotificationsCombobox->setCurrentIndex(showNotifications);
