@@ -38,6 +38,7 @@ class ConfigDialog : public QWidget
 
 public:
     ConfigDialog(int updateInterval,
+                 int postsPerPage,
                  int tabsPosition,
                  bool tabsMovable,
                  int publisherType,
@@ -47,6 +48,7 @@ public:
 
 signals:
     void configurationChanged(int newUpdateInterval,
+                              int newPostsPerPage,
                               int newTabsPosition,
                               bool newTabsMovable,
                               int newPublisherType,
@@ -63,6 +65,9 @@ private:
     QLabel *updateIntervalLabel;
     QSpinBox *updateIntervalSpinbox;
 
+    QLabel *postsPerPageLabel;
+    QSpinBox *postsPerPageSpinbox;
+
     QLabel *tabsPositionLabel;
     QComboBox *tabsPositionCombobox;
 
