@@ -38,9 +38,6 @@ ContactCard::ContactCard(PumpController *pumpController,
     // Get local file name, which is stored in base64 hash form
     QString avatarFilename = MiscHelpers::getCachedAvatarFilename(contactData.value("avatar"));
 
-    //qDebug() << "Avatar remote file:" << contactData.value("avatar");
-    //qDebug() << "Avatar local file:" << avatarFilename;
-
     QFile avatarFile(avatarFilename);
     if (avatarFile.exists(avatarFilename) && !contactData.value("avatar").isEmpty())
     {
@@ -56,7 +53,7 @@ ContactCard::ContactCard(PumpController *pumpController,
         avatarLabel->setPixmap(QIcon::fromTheme("user-identity").pixmap(48,48));
         qDebug() << "ContactCard: Using placeholder";
 
-        // Download avatar for next time
+        // Download avatar for next time (if empty, pumpController will do nothing)
         pController->getAvatar(contactData.value("avatar"));
     }
 
@@ -67,8 +64,10 @@ ContactCard::ContactCard(PumpController *pumpController,
     nameFont.setBold(true);
     nameFont.setUnderline(true);
 
+    QString contactNameString = contactData.value("name");
+
     nameLabel = new QLabel();
-    nameLabel->setText(contactData.value("name"));
+    nameLabel->setText(contactNameString);
     nameLabel->setFont(nameFont);
 
 
@@ -78,7 +77,25 @@ ContactCard::ContactCard(PumpController *pumpController,
     tmpLabel->setText(QString("<%1>\n").arg(contactID)
                       + tr("Hometown") + QString(": %2").arg(contactData.value("hometown")));
     tmpLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-    tmpLabel->setToolTip(contactData.value("bio")); // kinda tmp...
+
+
+    // Bio as tooltip for the whole contact card
+    QString contactBio = contactData.value("bio");
+    if (!contactBio.isEmpty())
+    {
+        // Make it rich text, so that it gets wordwrap
+        this->setToolTip("<b><u>"
+                         + tr("Bio for %1",
+                              "Abbreviation for Biography, but you can use the full word; %1=contact name")
+                           .arg(contactNameString)
+                         + "</u></b>"
+                         "<br><br>"
+                         + contactBio);
+    }
+    else
+    {
+        this->setToolTip(tr("No biography for %1", "%1=contact name").arg(contactNameString));
+    }
 
 
     centerLayout->addWidget(nameLabel);
