@@ -50,6 +50,8 @@ public:
     void clearFollowingListContents();
     void clearFollowersListContents();
 
+    void setTabLabels();
+
 signals:
 
 public slots:
@@ -70,10 +72,13 @@ private:
     QWidget *followingWidget;
     QVBoxLayout *followingLayout;
     QScrollArea *followingScrollArea;
+    int followingCount;
+
 
     QWidget *followersWidget;
     QVBoxLayout *followersLayout;
     QScrollArea *followersScrollArea;
+    int followersCount;
 
 
     PumpController *pController;
