@@ -57,51 +57,17 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     leftLayout->addSpacing(8);
 
     fullNameLabel = new QLabel("-Your-Name-");
-    leftLayout->addWidget(fullNameLabel); // FIXME: below avatar, temporarily
+    leftLayout->addWidget(fullNameLabel);
     leftLayout->addSpacing(16);
 
-    leftPanel = new QToolBox();
 
-    QLabel *tmpMeanwhileLabel = new QLabel("\n"
-                                           "- Somebody posted something.\n"
-                                           "- You followed a person.\n"
-                                           "- John Doe did something.\n"
-                                           "- etc.");
-    tmpMeanwhileLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+    leftPanel = new QToolBox(); // For now, only one "section"...
 
-    leftPanel->addItem(tmpMeanwhileLabel,
+    meanwhileFeed = new MinorFeed(pumpController);
+    leftPanel->addItem(meanwhileFeed,
                        QIcon::fromTheme("clock"),
                        tr("Meanwhile..."));
 
-
-    aspectListLabel = new QLabel("- <a href=\"list1\">List 1</a><br /><br />"
-                                 "- <a href=\"list2\">List 2</a><br /><br />"
-                                 "- <a href=\"list3\">List 3</a>");
-    aspectListLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
-    aspectListLabel->setContextMenuPolicy(Qt::NoContextMenu);
-    connect(aspectListLabel, SIGNAL(linkActivated(QString)),
-            pumpController, SLOT(setCurrentTimeline(QString)));
-    leftPanel->addItem(aspectListLabel,
-                       QIcon::fromTheme("view-list-text"),
-                       "&Lists");
-    QLabel *tmpTagsLabel = new QLabel("- <a href=\"tag1\">#tag1</a><br /><br />"
-                                      "- <a href=\"tag2\">#tag2</a><br /><br />"
-                                      "- <a href=\"tag3\">#tag3</a><br /><br />");
-    tmpTagsLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
-    tmpTagsLabel->setContextMenuPolicy(Qt::NoContextMenu);
-    connect(tmpTagsLabel, SIGNAL(linkActivated(QString)),
-            pumpController, SLOT(setCurrentTimeline(QString)));
-    leftPanel->addItem(tmpTagsLabel,
-                       QIcon::fromTheme("mail-tagged"),
-                       "#T&ags");
-    QLabel *tmpMoreLabel = new QLabel("- Activity\n\n"
-                                      "- @Mentions\n\n"
-                                      "");
-    tmpMoreLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
-    leftPanel->addItem(tmpMoreLabel,
-                       QIcon::fromTheme("kde"),
-                       "More...");
-
     leftLayout->addWidget(leftPanel);
 
     this->leftSideWidget->setLayout(leftLayout);
@@ -112,25 +78,52 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     rightSideWidget = new QWidget();
     rightLayout = new QVBoxLayout();
 
-    this->publisherType = settings.value("publisherType", 0).toInt(); // FIXME
+    this->publisherType = settings.value("publisherType", 0).toInt(); // TMP/FIXME?
     publisher = new Publisher(pumpController, publisherType);
 
 
-    timeLine = new TimeLine(pumpController, this);
-    timeLineScrollArea = new QScrollArea();
-    timeLineScrollArea->setWidget(timeLine);  // Make timeline scrollable
-    timeLineScrollArea->setWidgetResizable(true); // Seems to work in opposite way
-    connect(timeLine, SIGNAL(scrollToTop()),
-            this, SLOT(scrollTimelineToTop()));
+    mainTimeline = new TimeLine(TimeLine::TimelineTypeMain,
+                                pumpController,
+                                this);
+    mainTimelineScrollArea = new QScrollArea();
+    mainTimelineScrollArea->setWidget(mainTimeline);  // Make timeline scrollable
+    mainTimelineScrollArea->setWidgetResizable(true); // Seems to work in opposite way
+    connect(mainTimeline, SIGNAL(scrollToTop()),
+            this, SLOT(scrollMainTimelineToTop()));
 
-    connect(timeLine, SIGNAL(timelineRendered(int)),
+    connect(mainTimeline, SIGNAL(timelineRendered(int)),
             this, SLOT(notifyTimelineUpdate(int)));
 
 
-    messageList = new MessageList(this);
-    messageListScrollArea = new QScrollArea();
-    messageListScrollArea->setWidget(messageList);
-    messageListScrollArea->setWidgetResizable(true);
+    directTimeline = new TimeLine(TimeLine::TimelineTypeDirect,
+                                  pumpController,
+                                  this);
+    directTimelineScrollArea = new QScrollArea();
+    directTimelineScrollArea->setWidget(directTimeline);
+    directTimelineScrollArea->setWidgetResizable(true);
+    connect(directTimeline, SIGNAL(scrollToTop()),
+            this, SLOT(scrollDirectTimelineToTop()));
+
+
+    activityTimeline = new TimeLine(TimeLine::TimelineTypeActivity,
+                                    pumpController,
+                                    this);
+    activityTimelineScrollArea = new QScrollArea();
+    activityTimelineScrollArea->setWidget(activityTimeline);  // Make it scrollable
+    activityTimelineScrollArea->setWidgetResizable(true);
+    connect(activityTimeline, SIGNAL(scrollToTop()),
+            this, SLOT(scrollActivityTimelineToTop()));
+
+
+    favoritesTimeline = new TimeLine(TimeLine::TimelineTypeFavorites,
+                                     pumpController,
+                                     this);
+    favoritesTimelineScrollArea = new QScrollArea();
+    favoritesTimelineScrollArea->setWidget(favoritesTimeline);
+    favoritesTimelineScrollArea->setWidgetResizable(true);
+    connect(favoritesTimeline, SIGNAL(scrollToTop()),
+            this, SLOT(scrollFavoritesTimelineToTop()));
+
 
 
     // The contact list has its own tabs with its own scroll areas
@@ -138,31 +131,28 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
 
     tabWidget = new QTabWidget();
-    tabWidget->addTab(timeLineScrollArea,
+    tabWidget->addTab(mainTimelineScrollArea,
                       QIcon::fromTheme("view-list-details"),
                       tr("&Timeline"));
     tabWidget->setTabToolTip(0, tr("The main timeline"));
-    tabWidget->addTab(new QWidget(),
-                      QIcon::fromTheme("user-home"),
-                      tr("Activity"));
-    tabWidget->setTabToolTip(1, tr("Your own posts"));
-    tabWidget->addTab(new QWidget(),
-                      QIcon::fromTheme("folder-favorites"),
-                      tr("Favorites"));
-    tabWidget->setTabToolTip(2, tr("Your favorited posts"));
-    tabWidget->addTab(messageListScrollArea,
+    tabWidget->addTab(directTimelineScrollArea,
                       QIcon::fromTheme("mail-message"),
                       tr("&Messages"));
-    tabWidget->setTabToolTip(3, tr("Messages sent explicitly to you"));
+    tabWidget->setTabToolTip(1, tr("Messages sent explicitly to you"));
+    tabWidget->addTab(activityTimelineScrollArea,
+                      QIcon::fromTheme("user-home"),
+                      tr("&Activity"));
+    tabWidget->setTabToolTip(2, tr("Your own posts"));
+    tabWidget->addTab(favoritesTimelineScrollArea,
+                      QIcon::fromTheme("folder-favorites"),
+                      tr("Favorites"));
+    tabWidget->setTabToolTip(3, tr("Your favorited posts"));
     tabWidget->addTab(contactList,
                       QIcon::fromTheme("system-users"),
                       tr("&Contacts"));
     tabWidget->setTabToolTip(4, tr("The people you follow, and the ones who follow you"));
 
-    // TMP/FIXME: disable tabs which are not done yet
-    tabWidget->setTabEnabled(1, false); // activity
-    tabWidget->setTabEnabled(2, false); // favorites
-    tabWidget->setTabEnabled(3, false); // messages
+
 
     rightLayout->addWidget(publisher, 1); // stretch 1/10
     rightLayout->addWidget(tabWidget, 9); // stretch 9/10
@@ -211,28 +201,72 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
 
 
-    //////////////////////////////// Connections for PumpController
-    //// Incoming    
+    ///
+    //////////////////////////////// Connections for PumpController //////////
+    ///
+
     connect(pumpController, SIGNAL(profileReceived(QString,QString)),
             this, SLOT(updateProfileData(QString,QString)));
+
     connect(pumpController, SIGNAL(avatarPictureReceived(QByteArray,QUrl)),
             this, SLOT(storeAvatar(QByteArray,QUrl)));
     connect(pumpController, SIGNAL(imageReceived(QByteArray,QUrl)),
             this, SLOT(storeImage(QByteArray,QUrl)));
-    connect(pumpController, SIGNAL(aspectListReceived(QVariantList)),
-            this, SLOT(updateAspectList(QVariantList)));
 
-    connect(pumpController, SIGNAL(timeLineReceived(QVariantList,int,QString,QString)),
-            timeLine, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
 
+    // After receiving timeline contents, update corresponding timeline
+    connect(pumpController, SIGNAL(mainTimelineReceived(QVariantList,int,QString,QString)),
+            mainTimeline, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
+    connect(pumpController, SIGNAL(directTimelineReceived(QVariantList,int,QString,QString)),
+            directTimeline, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
+    connect(pumpController, SIGNAL(activityTimelineReceived(QVariantList,int,QString,QString)),
+            activityTimeline, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
+    connect(pumpController, SIGNAL(favoritesTimelineReceived(QVariantList,int,QString,QString)),
+            favoritesTimeline, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
+
+
+    // After sucessful posting, request updated timeline, with your post included
+    connect(pumpController, SIGNAL(postPublished()),
+            this, SLOT(updateTimelines()));
+    // FIXME: update other timelines, if appropiate
+    // (activity timeline, yes, direct timeline, no)
+
+
+    // After successful liking, update likes count
     connect(pumpController, SIGNAL(likesReceived(QVariantList,QString)),
-            timeLine, SLOT(setLikesInPost(QVariantList,QString)));
+            mainTimeline, SLOT(setLikesInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(likesReceived(QVariantList,QString)),
+            directTimeline, SLOT(setLikesInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(likesReceived(QVariantList,QString)),
+            activityTimeline, SLOT(setLikesInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(likesReceived(QVariantList,QString)),
+            favoritesTimeline, SLOT(setLikesInPost(QVariantList,QString)));
+    // After commenting successfully, refresh list of comments in that post
     connect(pumpController, SIGNAL(commentsReceived(QVariantList,QString)),
-            timeLine, SLOT(setCommentsInPost(QVariantList,QString)));
+            mainTimeline, SLOT(setCommentsInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(commentsReceived(QVariantList,QString)),
+            directTimeline, SLOT(setCommentsInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(commentsReceived(QVariantList,QString)),
+            activityTimeline, SLOT(setCommentsInPost(QVariantList,QString)));
+    connect(pumpController, SIGNAL(commentsReceived(QVariantList,QString)),
+            favoritesTimeline, SLOT(setCommentsInPost(QVariantList,QString)));
 
+
+    // After successful sharing....
+
+    // TODO***
+
+
+    // After receiving the minor feed ("Meanwhile"), update it
+    connect(pumpController, SIGNAL(minorFeedReceived(QVariantList)),
+            meanwhileFeed, SLOT(setFeedContents(QVariantList)));
+
+
+    // After receiving a contact list, update it
     connect(pumpController, SIGNAL(contactListReceived(QString,QVariantList)),
             contactList, SLOT(setContactListContents(QString,QVariantList)));
 
+    // Show notifications for events sent from pumpController
     connect(pumpController, SIGNAL(showNotification(QString)),
             fdNotifier, SLOT(showMessage(QString)));
 
@@ -366,26 +400,67 @@ void MainWindow::createMenus()
 {
     sessionMenu = new QMenu(tr("&Session"));
 
-    sessionUpdateTimelines = new QAction(QIcon::fromTheme("view-refresh"),
-                                         tr("&Update Timelines"),
+    sessionUpdateMainTimeline = new QAction(QIcon::fromTheme("view-refresh"),
+                                            tr("&Update Main Timeline"),
+                                            this);
+    sessionUpdateMainTimeline->setShortcut(QKeySequence::Refresh);
+    connect(sessionUpdateMainTimeline, SIGNAL(triggered()),
+            mainTimeline, SLOT(goToFirstPage()));
+    sessionMenu->addAction(sessionUpdateMainTimeline);
+
+    sessionUpdateDirectTimeline = new QAction(QIcon::fromTheme("view-refresh"),
+                                              tr("Update &Messages Timeline"),
+                                              this);
+    connect(sessionUpdateDirectTimeline, SIGNAL(triggered()),
+            directTimeline, SLOT(goToFirstPage()));
+    sessionMenu->addAction(sessionUpdateDirectTimeline);
+
+    sessionUpdateActivityTimeline = new QAction(QIcon::fromTheme("view-refresh"),
+                                                tr("Update &Activity Timeline"),
+                                                this);
+    connect(sessionUpdateActivityTimeline, SIGNAL(triggered()),
+            activityTimeline, SLOT(goToFirstPage()));
+    sessionMenu->addAction(sessionUpdateActivityTimeline);
+
+
+    sessionUpdateFavoritesTimeline = new QAction(QIcon::fromTheme("view-refresh"),
+                                                 tr("Update Favorites &Timeline"),
+                                                 this);
+    connect(sessionUpdateFavoritesTimeline, SIGNAL(triggered()),
+            favoritesTimeline, SLOT(goToFirstPage()));
+    sessionMenu->addAction(sessionUpdateFavoritesTimeline);
+
+
+    sessionUpdateMinorFeed = new QAction(QIcon::fromTheme("view-refresh"),
+                                         tr("Update Minor &Feed"),
                                          this);
-    sessionUpdateTimelines->setShortcut(QKeySequence::Refresh);
-    connect(sessionUpdateTimelines, SIGNAL(triggered()),
+    connect(sessionUpdateMinorFeed, SIGNAL(triggered()),
+            meanwhileFeed, SLOT(updateFeed()));
+    sessionMenu->addAction(sessionUpdateMinorFeed);
+
+
+    sessionUpdateAllTimelines = new QAction(QIcon::fromTheme("view-refresh"),
+                                         tr("Update All Timelines"),
+                                         this);
+    sessionUpdateAllTimelines->setShortcut(QKeySequence("Ctrl+F5"));
+    connect(sessionUpdateAllTimelines, SIGNAL(triggered()),
             this, SLOT(updateTimelines()));
-    sessionMenu->addAction(sessionUpdateTimelines);
+    sessionMenu->addAction(sessionUpdateAllTimelines);
 
+    sessionMenu->addSeparator();
 
-    sessionStartConversation = new QAction(QIcon::fromTheme("document-edit"),
+    sessionPostNote = new QAction(QIcon::fromTheme("document-edit"),
                                           tr("&Post a Note"),
                                           this);
-    sessionStartConversation->setShortcut(QKeySequence("Ctrl+N"));
-    connect(sessionStartConversation, SIGNAL(triggered()),
+    sessionPostNote->setShortcut(QKeySequence("Ctrl+N"));
+    connect(sessionPostNote, SIGNAL(triggered()),
             publisher, SLOT(setFullMode()));
-    connect(sessionStartConversation, SIGNAL(triggered()),
+    connect(sessionPostNote, SIGNAL(triggered()),
             this, SLOT(show()));  // show window, in case it's hidden and
-                                  // Start Conversation is used from tray menu
-    sessionMenu->addAction(sessionStartConversation);
+                                  // Post a Note is used from tray menu
+    sessionMenu->addAction(sessionPostNote);
 
+    sessionMenu->addSeparator();
 
     sessionQuit = new QAction(QIcon::fromTheme("application-exit"),
                               tr("&Quit"), this);
@@ -417,7 +492,6 @@ void MainWindow::createMenus()
 
     settingsAccount = new QAction(QIcon::fromTheme("user-properties"),
                                   tr("&Account Details"), this);
-    settingsAccount->setShortcut(QKeySequence("Ctrl+A"));
     connect(settingsAccount, SIGNAL(triggered()),
             accountDialog, SLOT(show()));
     settingsMenu->addAction(settingsAccount);
@@ -455,8 +529,8 @@ void MainWindow::createMenus()
                                tr("&Show Window"),
                                this, SLOT(show()));
     trayContextMenu->addSeparator();
-    trayContextMenu->addAction(sessionUpdateTimelines);
-    trayContextMenu->addAction(sessionStartConversation);
+    trayContextMenu->addAction(sessionUpdateMainTimeline);
+    trayContextMenu->addAction(sessionPostNote);
     trayContextMenu->addAction(settingsConfigure);
     trayContextMenu->addAction(helpAbout);
     trayContextMenu->addSeparator();
@@ -658,16 +732,34 @@ void MainWindow::trayControl(QSystemTrayIcon::ActivationReason reason)
 {
     qDebug() << "Tray icon activation reason:" << reason;
 
-    if (reason == QSystemTrayIcon::Context)
+    if (reason != QSystemTrayIcon::Context) // Simple "main button" click in icon
     {
-        qDebug() << "Tray icon context menu here";
-    }
-    else
-    {
-         // Hide or show the main window
-        this->setVisible(this->isHidden()); // this->setShown() deprecated on Qt5
-                                            // replace with setVisible()
-        qDebug() << "trayControl() - MainWindow hidden?" << this->isHidden();
+        /*
+        qDebug() << "trayControl()";
+        qDebug() << "isHidden?"    << this->isHidden();
+        qDebug() << "isVisible?"   << this->isVisible();
+        qDebug() << "isMinimized?" << this->isMinimized();
+        qDebug() << "hasFocus?"    << this->hasFocus();
+        */
+
+        // Hide or show the main window
+        if (this->isMinimized())
+        {
+            // hide and show, because raise() wouln't work
+            this->hide();
+            this->showNormal();
+            qDebug() << "RAISING!";
+        }
+        else if (this->isHidden())
+        {
+            this->show();
+            qDebug() << "SHOWING";
+        }
+        else
+        {
+            this->hide();
+            qDebug() << "HIDING";
+        }
     }
 }
 
@@ -718,52 +810,40 @@ void MainWindow::updateProfileData(QString fullName, QString avatarURL)
 
 
 
-
-void MainWindow::updateAspectList(QVariantList aspectList)
-{
-    QString aspectsString;
-
-    foreach (QVariant aspect, aspectList)
-    {
-        if (aspect.type() == QVariant::Map)
-        {
-            QVariantMap aspectMap = aspect.toMap();
-
-            QString aspectName = aspectMap.value("aspect").toMap().value("name").toString();
-            QString aspectId = aspectMap.value("aspect").toMap().value("id").toString();
-
-            aspectsString.append(QString("- <a href=\"aspect:/%2\"><b>%1</b> (%2)</a><br /><br />")
-                                .arg(aspectName)
-                                .arg(aspectId));
-        }
-        else
-        {
-            qDebug() << "Expected a Map, got something else";
-        }
-    }
-
-    this->aspectListLabel->setText(aspectsString);
-
-    qDebug() << "Updated aspect list from server:" << aspectsString;
-}
-
-
-
 void MainWindow::updateTimelines()
 {
-    pumpController->getTimeline(0);
+    mainTimeline->goToFirstPage();
     // received timeline data will come in a SIGNAL()
 
+    meanwhileFeed->updateFeed(); // kinda TMP / FIXME
 
     qDebug() << "Updated timelines by menu or after:" << this->updateInterval << "min";
 }
 
 
 
-void MainWindow::scrollTimelineToTop()
+void MainWindow::scrollMainTimelineToTop()
 {
-    this->timeLineScrollArea->verticalScrollBar()->setValue(0);
-    this->timeLineScrollArea->horizontalScrollBar()->setValue(0);
+    this->mainTimelineScrollArea->verticalScrollBar()->setValue(0);
+    this->mainTimelineScrollArea->horizontalScrollBar()->setValue(0);
+}
+
+void MainWindow::scrollDirectTimelineToTop()
+{
+    this->directTimelineScrollArea->verticalScrollBar()->setValue(0);
+    this->directTimelineScrollArea->horizontalScrollBar()->setValue(0);
+}
+
+void MainWindow::scrollActivityTimelineToTop()
+{
+    this->activityTimelineScrollArea->verticalScrollBar()->setValue(0);
+    this->activityTimelineScrollArea->horizontalScrollBar()->setValue(0);
+}
+
+void MainWindow::scrollFavoritesTimelineToTop()
+{
+    this->favoritesTimelineScrollArea->verticalScrollBar()->setValue(0);
+    this->favoritesTimelineScrollArea->horizontalScrollBar()->setValue(0);
 }
 
 
@@ -772,12 +852,22 @@ void MainWindow::notifyTimelineUpdate(int newPostCount)
 {
     if (newPostCount > 0)
     {
-        this->setStatusBarMessage(tr("Timeline updated. There are %1 new posts.").arg(newPostCount));
+        QString newPostsString;
+        if (newPostCount == 1)
+        {
+            newPostsString = tr("There is 1 new post.");
+        }
+        else
+        {
+            newPostsString = tr("There are %1 new posts.").arg(newPostCount);
+        }
+        this->setStatusBarMessage(tr("Timeline updated.") + " " + newPostsString);
+
 
         if (this->showNotifications != 2) // If some type of notifications are enabled
         {
-            this->fdNotifier->showMessage(tr("Timeline updated at %1.\nThere are %2 new posts.")
-                                          .arg(QTime::currentTime().toString()).arg(newPostCount));
+            this->fdNotifier->showMessage(tr("Timeline updated at %1.").arg(QTime::currentTime().toString())
+                                        + " " + newPostsString);
         }
     }
     else
@@ -872,7 +962,7 @@ void MainWindow::toggleSidePanel(bool shown)
     if (!this->publisher->hasFocus())
     {
         // FIXME: check if publisher's composer has focus!
-        this->timeLine->setFocus();
+        this->mainTimeline->setFocus();
     }
 
     qDebug() << "Showing side panel:" << shown;
@@ -903,18 +993,19 @@ void MainWindow::visitWebSite()
 void MainWindow::aboutDianara()
 {
     QMessageBox::about(this, tr("About Dianara"),
-                       "<b>Dianara v0.6</b><br>"
-                       "Copyright 2012-2013  JanKusanagi<br>"
+                       "<b>Dianara v0.7</b><br>"
+                       "Copyright 2012-2013  JanKusanagi<br><br>"
                        "<a href=\"http://jancoding.wordpress.com/dianara\">"
-                       "http://jancoding.wordpress.com/dianara</a><br>"
-                       "<a href=\"http://dianara.nongnu.org\">"
-                       "http://dianara.nongnu.org</a><br><br>"
+                       "http://jancoding.wordpress.com/dianara</a><br><br>"
+
                        + tr("Dianara is a pump.io social networking client.")
                        + "<br><br>"
-                       + tr("With Dianara you can see your timeline, create new posts, "
-                            "like/comment/share/delete posts, "
-                            "see your contacts and follow new people.")
+
+                       + tr("With Dianara you can see your timelines, create new posts, "
+                            "upload pictures, interact with posts, manage "
+                            "your contacts and follow new people.")
                        + "<br><br><br>"
+
                        + tr("English translation by JanKusanagi.",
                             "TRANSLATORS: Change this with your language and name ;)"));
 
