@@ -58,8 +58,8 @@
 #include "publisher.h"
 #include "timeline.h"
 #include "post.h"
-#include "messagelist.h"
 #include "contactlist.h"
+#include "minorfeed.h"
 
 
 class MainWindow : public QMainWindow
@@ -91,10 +91,13 @@ public slots:
     void showTrayFallbackMessage(QString message);
 
     void updateProfileData(QString fullName, QString avatarURL);
-    void updateAspectList(QVariantList aspectList);
 
     void updateTimelines();
-    void scrollTimelineToTop();
+
+    void scrollMainTimelineToTop();
+    void scrollDirectTimelineToTop();
+    void scrollActivityTimelineToTop();
+    void scrollFavoritesTimelineToTop();
     void notifyTimelineUpdate(int newPostCount);
 
     void storeAvatar(QByteArray avatarData, QUrl avatarURL);
@@ -115,6 +118,7 @@ protected:
 
 
 private:
+    // Menus
     QMenu *sessionMenu;
     QMenu *viewMenu;
     QMenu *settingsMenu;
@@ -122,8 +126,13 @@ private:
 
     QMenu *trayContextMenu;
 
-    QAction *sessionUpdateTimelines;
-    QAction *sessionStartConversation;
+    QAction *sessionUpdateMainTimeline;
+    QAction *sessionUpdateDirectTimeline;
+    QAction *sessionUpdateActivityTimeline;
+    QAction *sessionUpdateFavoritesTimeline;
+    QAction *sessionUpdateMinorFeed;
+    QAction *sessionUpdateAllTimelines;
+    QAction *sessionPostNote;
     QAction *sessionQuit;
     QAction *viewSidePanel;
     QAction *settingsAccount;
@@ -158,11 +167,22 @@ private:
     PumpController *pumpController;
     FDNotifications *fdNotifier;
 
-    TimeLine *timeLine;
+    MinorFeed *meanwhileFeed;
+
     int postsPerPage;
-    QScrollArea *timeLineScrollArea;
-    MessageList *messageList;
-    QScrollArea *messageListScrollArea;
+    TimeLine *mainTimeline;
+    QScrollArea *mainTimelineScrollArea;
+
+    TimeLine *directTimeline;
+    QScrollArea *directTimelineScrollArea;
+
+    TimeLine *activityTimeline;
+    QScrollArea *activityTimelineScrollArea;
+
+    TimeLine *favoritesTimeline;
+    QScrollArea *favoritesTimelineScrollArea;
+
+
     ContactList *contactList;
 
     Publisher *publisher;
@@ -183,7 +203,6 @@ private:
     QString userAuth;
     QByteArray ownAvatarData;
 
-    QLabel *aspectListLabel;
 
 };
 
