@@ -24,6 +24,7 @@
 #include <QObject>
 #include <QString>
 #include <QRegExp>
+#include <QStringList>
 
 #include <QDesktopServices> // TMP: getting avatar directory
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
@@ -38,9 +39,12 @@ class MiscHelpers : public QObject
 
 public:
     explicit MiscHelpers(QObject *parent = 0);
+
     static QString getCachedAvatarFilename(QString url);
     static QString getCachedImageFilename(QString url);
 
+    static QStringList htmlWithReplacedImages(QString originalHtml);
+
 signals:
 
 public slots:
