@@ -22,7 +22,9 @@
 
 Post::Post(PumpController *pumpController,
            QMap<QString,QString> postData,
-           QVariantList commentsVariantList,
+           QVariantList lastLikesVariantList,
+           QVariantList lastCommentsVariantList,
+           QVariantList lastSharesVariantList,
            QWidget *parent)  :  QFrame(parent)
 {
     this->pController = pumpController;
@@ -35,20 +37,47 @@ Post::Post(PumpController *pumpController,
 
 
     leftColumnLayout = new QVBoxLayout();
-    leftColumnLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);
 
 
     this->postID = postData.value("id");
     this->postType = postData.value("postType");
 
 
+    QFont detailsFont;
+    detailsFont.setPointSize(detailsFont.pointSize() - 2); // FIXME: check size first
+
+
     /////////////////////////////////////////////////// Left column, post Meta info
 
+    // Is the post a reshare? Indicate who shared it
+    postIsSharedLabel = new QLabel();
+    if (postData.value("postIsShared") == "true")
+    {
+        postIsSharedLabel->setText(tr("Via %1").arg(postData.value("sharedBy")));
+        postIsSharedLabel->setWordWrap(true);
+        postIsSharedLabel->setAlignment(Qt::AlignCenter);
+        postIsSharedLabel->setFont(detailsFont);
+        postIsSharedLabel->setForegroundRole(QPalette::Background);
+        postIsSharedLabel->setBackgroundRole(QPalette::Foreground);
+        postIsSharedLabel->setAutoFillBackground(true);
+        postIsSharedLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
+        leftColumnLayout->addWidget(postIsSharedLabel);
+    }
+    leftColumnLayout->addSpacing(4);
+
+
     QString authorTooltipInfo = postData.value("authorId");
-    authorTooltipInfo.append("\n");
-    authorTooltipInfo.append(tr("Hometown") + ": " + postData.value("authorHometown"));
-    authorTooltipInfo.append("\n\n");
-    authorTooltipInfo.append(postData.value("authorBio"));
+    if (!postData.value("authorHometown").isEmpty())
+    {
+        authorTooltipInfo.append("<br>");
+        authorTooltipInfo.append(tr("Hometown") + ": " + postData.value("authorHometown"));
+        authorTooltipInfo.append("<br>");
+    }
+    if (!postData.value("authorBio").isEmpty())
+    {
+        authorTooltipInfo.append("<br>");
+        authorTooltipInfo.append(postData.value("authorBio"));
+    }
 
     if (postData.value("authorId") == pController->currentUserID())
     {
@@ -66,10 +95,8 @@ Post::Post(PumpController *pumpController,
     // Get local file name, which is stored in base64 hash form
     QString avatarFilename = MiscHelpers::getCachedAvatarFilename(postData.value("authorAvatar"));
 
-    //qDebug() << "Avatar remote file:" << postData.value("authorAvatar");
-    //qDebug() << "Avatar local file:" << avatarFilename;
 
-    if (QFile::exists(avatarFilename))
+    if (QFile::exists(avatarFilename) && !postData.value("authorAvatar").isEmpty())
     {
         // Load avatar if already cached
         postAuthorAvatar->setPixmap(QPixmap(avatarFilename)
@@ -90,6 +117,7 @@ Post::Post(PumpController *pumpController,
     postAuthorAvatar->setToolTip(authorTooltipInfo);
 
     leftColumnLayout->addWidget(postAuthorAvatar);
+    leftColumnLayout->addSpacing(2);
 
 
     QFont authorFont;
@@ -109,19 +137,26 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addSpacing(4);
 
 
-    QFont detailsFont;
-    detailsFont.setPointSize(detailsFont.pointSize() - 2); // FIXME: check size first
 
     // Post creation time
     QString createdAt = postData.value("createdAt");
     // Format is "Combined date and time in UTC", like "2012-02-07T01:32:02Z"
-    // As specified in ISO 8601 http://en.wikipedia.org/wiki/ISO_8601
+    // as specified in ISO 8601 http://en.wikipedia.org/wiki/ISO_8601
     QString exactPostTime = Timestamp::localTimeDate(createdAt);
     createdAt = Timestamp::fuzzyTime(createdAt);
+    QString generator = postData.value("generator");
 
     postCreatedAt = new QLabel(createdAt);
-    postCreatedAt->setToolTip(tr("Posted on %1, using %2","1=Date, 2=Program used")
-                              .arg(exactPostTime).arg(postData.value("generator")));
+    if (generator.isEmpty())
+    {
+        postCreatedAt->setToolTip(tr("Posted on %1","1=Date")
+                                  .arg(exactPostTime));
+    }
+    else
+    {
+        postCreatedAt->setToolTip(tr("Posted on %1, using %2","1=Date, 2=Program used")
+                                  .arg(exactPostTime).arg(generator));
+    }
     postCreatedAt->setTextFormat(Qt::PlainText);
     postCreatedAt->setWordWrap(true);
     postCreatedAt->setAlignment(Qt::AlignLeft | Qt::AlignTop);
@@ -129,7 +164,20 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addWidget(postCreatedAt);
     leftColumnLayout->addSpacing(4);
 
-    openInBrowserLabel = new QLabel("<br><a href=\"" + postData.value("url") + "\">Open &#9658;</a>");
+
+    postToLabel = new QLabel(tr("To") + ": " + postData.value("to"));
+    postToLabel->setWordWrap(true);
+    postToLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postToLabel);
+
+    postCCLabel = new QLabel(tr("CC") + ": " + postData.value("cc"));
+    postCCLabel->setWordWrap(true);
+    postCCLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postCCLabel);
+    leftColumnLayout->addSpacing(4);
+
+
+    openInBrowserLabel = new QLabel("<a href=\"" + postData.value("url") + "\">" + tr("Open") +" &#9658;</a>");
     openInBrowserLabel->setToolTip(tr("Open this post in your web browser"));
     openInBrowserLabel->setOpenExternalLinks(true);
     openInBrowserLabel->setFont(detailsFont);
@@ -138,94 +186,52 @@ Post::Post(PumpController *pumpController,
     leftColumnLayout->addSpacing(12);
 
 
-
-
-    // Is the post a reshare? // FIXME
-    if (postData.value("postIsShared") == "true")
-    {
-        postTypeLabel = new QLabel(tr("Reshared post"));
-        postTypeLabel->setWordWrap(true);
-        postTypeLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-        postTypeLabel->setFont(detailsFont);
-        leftColumnLayout->addWidget(postTypeLabel);
-/*
-        postRootAuthor = new QLabel(postData.value("rootAuthor"));
-        postRootAuthor->setTextFormat(Qt::PlainText);
-        postRootAuthor->setWordWrap(true);
-        postRootAuthor->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-        postRootAuthor->setFont(detailsFont);
-        leftColumnLayout->addWidget(postRootAuthor);*/
-    }
-
-
     this->postLikesURL = postData.value("likesURL");
     this->postCommentsURL = postData.value("commentsURL");
 
 
-
-    postLikesCount = new QLabel();
-    QString likesCount = postData.value("likesCount");
-    if (likesCount != "0")
-    {
-        if (likesCount == "1")
-        {
-            postLikesCount->setText(QString::fromUtf8("\342\231\245 ") // heart symbol
-                                  + tr("1 like"));
-        }
-        else
-        {
-            postLikesCount->setText(QString::fromUtf8("\342\231\245 ") // heart symbol
-                                  + tr("%1 likes").arg(likesCount));
-        }
-        postLikesCount->setWordWrap(true);
-        postLikesCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-        postLikesCount->setFont(detailsFont);
-        leftColumnLayout->addWidget(postLikesCount);
-    }
+    // Set this QLabels with parent=this, since we're gonna hide them right away
+    // They'll be reparented to the layout, but now having a parent before that
+    // would cause visual glitches
+    postLikesCountLabel = new QLabel(this);
+    postLikesCountLabel->setWordWrap(true);
+    postLikesCountLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    postLikesCountLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postLikesCountLabel);
+    postLikesCountLabel->hide();
+    int likesCount = postData.value("likesCount").toInt();
+    this->setLikesLabel(likesCount);
 
 
-    postCommentsCount = new QLabel();
-    QString commentsCount = postData.value("commentsCount");
-    if (commentsCount != "0")
-    {
-        if (commentsCount == "1")
-        {
-            postCommentsCount->setText(tr("1 comment"));
-        }
-        else
-        {
-            postCommentsCount->setText(tr("%1 comments").arg(commentsCount));
-        }
-        postCommentsCount->setWordWrap(true);
-        postCommentsCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-        postCommentsCount->setFont(detailsFont);
-        leftColumnLayout->addWidget(postCommentsCount);
-    }
-
-    postResharesCount = new QLabel();
-    QString resharesCount = postData.value("resharesCount");
-    if (resharesCount != "0")
-    {
-        if (resharesCount == "1")
-        {
-            postResharesCount->setText(QString::fromUtf8("\342\231\272 ") // recycle symbol
-                                      + tr("Shared one time"));
-        }
-        else
-        {
-            postResharesCount->setText(QString::fromUtf8("\342\231\272 ") // recycle symbol
-                                      + tr("Shared %1 times").arg(resharesCount));
-        }
-        postResharesCount->setWordWrap(true);
-        postResharesCount->setAlignment(Qt::AlignLeft | Qt::AlignTop);
-        postResharesCount->setFont(detailsFont);
-        leftColumnLayout->addWidget(postResharesCount);
-    }
+    postCommentsCountLabel = new QLabel(this);
+    postCommentsCountLabel->setWordWrap(true);
+    postCommentsCountLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    postCommentsCountLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postCommentsCountLabel);
+    postCommentsCountLabel->hide();
+    int commentsCount = postData.value("commentsCount").toInt();
+    this->setCommentsLabel(commentsCount);
 
 
+    postResharesCountLabel = new QLabel(this);
+    postResharesCountLabel->setWordWrap(true);
+    postResharesCountLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    postResharesCountLabel->setFont(detailsFont);
+    leftColumnLayout->addWidget(postResharesCountLabel);
+    postResharesCountLabel->hide();
+    int resharesCount = postData.value("resharesCount").toInt();
+    this->setSharesLabel(resharesCount);
 
 
-    ////////////////////////////////////////////////// Right column, content
+    // Try to use all remaining space, aligning
+    // all previous widgets nicely at the top
+    // leftColumnLayout->setAlignment(Qt::AlignTop) caused glitches
+    leftColumnLayout->addStretch(1);
+
+
+    //////////////////////////////////////////////////////////////////////////
+    //////////////////////////////////////////////////////////////////////////
+    //////////////////////////////////////////////////// Right column, content
 
     rightColumnLayout = new QVBoxLayout();
     rightColumnLayout->setAlignment(Qt::AlignRight | Qt::AlignTop);
@@ -246,6 +252,22 @@ Post::Post(PumpController *pumpController,
     connect(postText, SIGNAL(highlighted(QUrl)),
             this, SLOT(showHighlightedURL(QUrl)));
 
+    // Add a label in the bottom right corner, to show where links go
+    postTextLayout = new QHBoxLayout();
+
+    highlightedUrlLabel = new QLabel();
+    highlightedUrlLabel->setAlignment(Qt::AlignLeft);
+    highlightedUrlLabel->setFont(detailsFont);
+    highlightedUrlLabel->setForegroundRole(QPalette::ToolTipText); // use Tooltip colors
+    highlightedUrlLabel->setBackgroundRole(QPalette::ToolTipBase);
+    highlightedUrlLabel->setAutoFillBackground(true);
+    highlightedUrlLabel->setWordWrap(true);
+    highlightedUrlLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
+
+    postTextLayout->addWidget(highlightedUrlLabel, 0, Qt::AlignRight | Qt::AlignBottom);
+    highlightedUrlLabel->hide(); // Hidden initially
+    postText->setLayout(postTextLayout);
+
 
     // Like, comment, share buttons
     likeButton = new QPushButton(QIcon::fromTheme("emblem-favorite"), "");
@@ -317,15 +339,41 @@ Post::Post(PumpController *pumpController,
         {
             postTitle = "<h3>" + postImageTitle + "</h3><br>";
         }
-        postImage = "<a href=\"image:/" + MiscHelpers::getCachedImageFilename(postImage) + "\"><img src=\"" + MiscHelpers::getCachedImageFilename(postImage) + "\" width=\"320\"></img></a><br><br>";
+        postImage = "<a href=\"image:/" + MiscHelpers::getCachedImageFilename(postImage) + "\">"
+                    "<img src=\"" + MiscHelpers::getCachedImageFilename(postImage) + "\" width=\"320\" />"
+                    "</a>"
+                    "<br><br>";
     }
 
 
+    QStringList postTextImageList = MiscHelpers::htmlWithReplacedImages(postData.value("text"));
+    QString postTextContents = postTextImageList.takeAt(0);
+
 
     // Add the text content of the post
     postText->setHtml(postTitle
                     + postImage
-                    + postData.value("text"));
+                    + postTextContents);
+
+
+    // If the image list is not empty, get them (unless they're cached already)
+    if (!postTextImageList.isEmpty())
+    {
+        qDebug() << "Post has" << postTextImageList.size() << "images included...";
+
+        foreach (QString imageUrl, postTextImageList)
+        {
+            if (QFile::exists(MiscHelpers::getCachedImageFilename(imageUrl)))
+            {
+                qDebug() << "Image already cached, ignoring";
+            }
+            else
+            {
+                qDebug() << "Image not cached, downloading";
+                this->pController->getImage(imageUrl);
+            }
+        }
+    }
 
 
     int postLines = postText->document()->lineCount() + 1;
@@ -334,11 +382,11 @@ Post::Post(PumpController *pumpController,
     int postMinimumHeight = postLines * 2 * postText->fontInfo().pixelSize();
     if (postType == "image")
     {
-        postMinimumHeight += 200;
+        postMinimumHeight += 100;
     }
-    if (postMinimumHeight > 400)  // Don't allow a post to have a minimum height > 400 px
+    if (postMinimumHeight > 100)  // Don't allow a post to have a minimum height > 200 px
     {
-        postMinimumHeight = 400;  // Scrollbars will appear
+        postMinimumHeight = 100;  // Scrollbars will appear
         qDebug() << "Post would require more space: scrollbars might appear";
     }
     qDebug() << "Post minimum height:" << postMinimumHeight;
@@ -346,8 +394,8 @@ Post::Post(PumpController *pumpController,
 
 
     // NSFW button, to replace #nsfw posts until clicked
-    nsfwCoveringButton = new QPushButton(tr("This post has been marked Not Safe For Work (#NSFW).\n"
-                                            "Click here to show it."));
+    nsfwCoveringButton = new QPushButton("This post has been marked Not Safe For Work (#NSFW).\n"
+                                         "Click here to show it.");
     nsfwCoveringButton->setSizePolicy(QSizePolicy::MinimumExpanding,
                                       QSizePolicy::MinimumExpanding);
     connect(nsfwCoveringButton, SIGNAL(clicked()),
@@ -362,19 +410,20 @@ Post::Post(PumpController *pumpController,
             this, SLOT(getAllComments()));
 
 
-    rightColumnLayout->addWidget(postText,  7);
-    rightColumnLayout->addLayout(buttonsLayout);
-    rightColumnLayout->addWidget(commenter, 1);
+    rightColumnLayout->addWidget(postText,      2);
+    rightColumnLayout->addLayout(buttonsLayout, 0);
+    rightColumnLayout->addWidget(commenter,     0);
 
 
-    // Set the initial comments (4 most recent)
-    this->commenter->setComments(commentsVariantList);
-
+    // Set the initial likes, comments and shares (4 most recent)
+    this->setLikes(lastLikesVariantList);
+    this->commenter->setComments(lastCommentsVariantList);
+    this->setShares(lastSharesVariantList);
 
 
     mainLayout = new QHBoxLayout();
-    mainLayout->addLayout(leftColumnLayout,  1); // stretch 1/6
-    mainLayout->addLayout(rightColumnLayout, 5); // stretch 5/6
+    mainLayout->addLayout(leftColumnLayout,  2); // stretch 2/10
+    mainLayout->addLayout(rightColumnLayout, 8); // stretch 8/10
 
     if (postData.value("nsfw") == "true")
     {
@@ -417,18 +466,60 @@ void Post::setLikes(QVariantList likesList)
 {
     if (likesList.size() > 0)
     {
-        QString likesString;
+        QString peopleString;
         foreach (QVariant likesMap, likesList)
         {
-            likesString = likesMap.toMap().value("displayName").toString() + ", "
-                    + likesString;
+            peopleString = likesMap.toMap().value("displayName").toString() + ", "
+                    + peopleString;
         }
 
         // Remove last comma and add "like this"
-        likesString.remove(-2, 2); // Last 2 characters
-        likesString.append(" like this");
+        peopleString.remove(-2, 2); // Last 2 characters
 
-        this->postLikesCount->setToolTip(likesString);
+        QString likesString;
+        if (likesList.size() == 1)
+        {
+            likesString = tr("%1 likes this", "One person").arg(peopleString);
+        }
+        else
+        {
+            likesString = tr("%1 like this", "More than one person").arg(peopleString);
+        }
+        likesString.append("<a></a>"); // Turn the label into rich text
+
+        this->postLikesCountLabel->setToolTip(likesString);
+    }
+
+    // TMP/FIXME this can set the number to lower than initially set
+    // if called with the "initial" up-to-4 likes list that comes with the post
+    this->setLikesLabel(likesList.size());
+}
+
+
+/*
+ * Update the "NUMBER likes" label in left side
+ *
+ */
+void Post::setLikesLabel(int likesCount)
+{
+    if (likesCount != 0)
+    {
+        if (likesCount == 1)
+        {
+            postLikesCountLabel->setText(QString::fromUtf8("\342\231\245 ") // heart symbol
+                                  + tr("1 like"));
+        }
+        else
+        {
+            postLikesCountLabel->setText(QString::fromUtf8("\342\231\245 ") // heart symbol
+                                  + tr("%1 likes").arg(likesCount));
+        }
+        postLikesCountLabel->show();
+    }
+    else
+    {
+        postLikesCountLabel->clear();
+        postLikesCountLabel->hide();
     }
 }
 
@@ -450,6 +541,36 @@ QString Post::commentsURL()
 void Post::setComments(QVariantList commentsList)
 {
     this->commenter->setComments(commentsList);
+
+    // update number of comments in left side counter
+    this->setCommentsLabel(commentsList.size());
+}
+
+/*
+ * Update the "NUMBER comments" label in left side
+ *
+ */
+void Post::setCommentsLabel(int commentsCount)
+{
+    if (commentsCount != 0)
+    {
+        if (commentsCount == 1)
+        {
+            postCommentsCountLabel->setText(QString::fromUtf8("\342\234\215 ") // writing hand
+                                      + tr("1 comment"));
+        }
+        else
+        {
+            postCommentsCountLabel->setText(QString::fromUtf8("\342\234\215 ") // writing hand
+                                      + tr("%1 comments").arg(commentsCount));
+        }
+        postCommentsCountLabel->show();
+    }
+    else
+    {
+        postCommentsCountLabel->clear();
+        postCommentsCountLabel->hide();
+    }
 }
 
 
@@ -463,13 +584,67 @@ QString Post::sharesURL()
     return this->postSharesURL;
 }
 
+
 /*
  * Update the tooltip for "%NUMBER shares" with names
  *
  */
 void Post::setShares(QVariantList sharesList)
 {
+    if (sharesList.size() > 0)
+    {
+        QString peopleString;
+        foreach (QVariant sharesMap, sharesList)
+        {
+            peopleString = sharesMap.toMap().value("displayName").toString() + ", "
+                    + peopleString;
+        }
 
+        // Remove last comma and add "shared this"
+        peopleString.remove(-2, 2); // Last 2 characters
+
+        QString sharesString;
+        if (sharesList.size() == 1)
+        {
+            sharesString = tr("%1 shared this", "%1 = One person name").arg(peopleString);
+        }
+        else
+        {
+            sharesString = tr("%1 shared this", "%1 = Names for more than one person").arg(peopleString);
+        }
+        sharesString.append("<a></a>");  // So that the label is rich text
+
+        this->postResharesCountLabel->setToolTip(sharesString);
+    }
+
+    // TMP/FIXME this can set the number to lower than initially set
+    // if called with the "initial" up-to-4 shares list that comes with the post
+    this->setSharesLabel(sharesList.size());
+}
+
+
+void Post::setSharesLabel(int resharesCount)
+{
+    if (resharesCount != 0)
+    {
+        if (resharesCount == 1)
+        {
+            postResharesCountLabel->setText(QString::fromUtf8("\342\231\273 ") // recycle symbol
+                                      + tr("Shared one time"));
+        }
+        else
+        {
+            postResharesCountLabel->setText(QString::fromUtf8("\342\231\273 ") // recycle symbol
+                                      + tr("Shared %1 times").arg(resharesCount));
+        }
+
+        postResharesCountLabel->show();
+    }
+    else
+    {
+        postResharesCountLabel->clear();
+        postResharesCountLabel->hide();
+    }
 }
 
 
@@ -514,6 +689,9 @@ void Post::likePost(bool like)
     this->pController->likePost(this->postID, this->postType, like);
 
     this->fixLikeButton(like ? "true" : "false");
+
+    connect(pController, SIGNAL(likeSet()),
+            this, SLOT(getAllLikes()));
 }
 
 /*
@@ -537,6 +715,15 @@ void Post::fixLikeButton(QString state)
 }
 
 
+void Post::getAllLikes()
+{
+    disconnect(pController, SIGNAL(likeSet()),
+               this, SLOT(getAllLikes()));
+
+    this->pController->getPostLikes(this->postLikesURL);
+}
+
+
 /*
  * Make the commenter widget visible, so user can type the comment
  *
@@ -564,9 +751,6 @@ void Post::sendComment(QString comment)
 void Post::getAllComments()
 {
     this->pController->getPostComments(this->postCommentsURL);
-
-
-    this->pController->getPostLikes(this->postLikesURL); // tmp
 }
 
 
@@ -599,7 +783,7 @@ void Post::sharePost()
  */
 void Post::deletePost()
 {
-    int confirmation = QMessageBox::question(this, tr("Delete post?"),
+    int confirmation = QMessageBox::question(this, tr("WARNING: Delete post?"),
                                              tr("Are you sure you want to delete this post?"),
                                              tr("&Yes, delete it"), tr("&No"));
 
@@ -644,6 +828,20 @@ void Post::showHighlightedURL(QUrl url)
     QString urlString = url.toString();
     if (!urlString.isEmpty())
     {
-        qDebug() << "Highlighted url:" << urlString;
+        if (urlString.startsWith("image://")) // Own image:// URL
+        {
+            this->highlightedUrlLabel->setText(tr("Click the image to see it in full size"));
+        }
+        else  // Normal URL
+        {
+            this->highlightedUrlLabel->setText(tr("Link to: %1").arg(urlString));
+            qDebug() << "Highlighted url:" << urlString;
+        }
+        this->highlightedUrlLabel->show();
+    }
+    else
+    {
+        this->highlightedUrlLabel->clear();
+        this->highlightedUrlLabel->hide();
     }
 }
