@@ -24,42 +24,66 @@
 #include <QWidget>
 #include <QGridLayout>
 #include <QLabel>
-#include <QTextEdit>
+#include <QLineEdit>
 #include <QPushButton>
 #include <QMenu>
+#include <QFileDialog>
+#include <QImageReader>
+#include <QMessageBox>
 
 #include <QDebug>
 
 #include "composer.h"
+#include "pumpcontroller.h"
 
 class Publisher : public QWidget
 {
     Q_OBJECT
 
 public:
-    explicit Publisher(int publisherType, QWidget *parent = 0);
+    explicit Publisher(PumpController *pumpController,
+                       int publisherType,
+                       QWidget *parent = 0);
+    ~Publisher();
+
+    void setEmptyPictureData();
+
+    QMap<QString,QString> getAudienceMap();
 
 
 signals:
-    void sharePost(QString postText); // TODO: params like Lists/Public, pics...
+
 
 public slots:
     void setMinimumMode();
     void setFullMode();
-    void share();
+    void setPictureMode();
+
+    void findPictureFile();
+
+    void sendPost();
 
 private:
     QGridLayout *mainLayout;
 
+    QLineEdit *titleLineEdit;
+    QLabel *pictureLabel;
+    QPushButton *selectPictureButton;
+
     Composer *composerBox;
 
     QMenu *aspectList;
     QPushButton *aspectSelector;
 
     QPushButton *pictureButton;
-    QPushButton *shareButton;
+    QPushButton *postButton;
 
 
+    QString pictureFilename;
+    QString pictureContentType;
+
+
+    PumpController *pController;
 };
 
 #endif // PUBLISHER_H
