@@ -22,9 +22,10 @@
 
 PumpController::PumpController(QObject *parent) :  QObject(parent)
 {
-    this->userAgentString = "Dianara/0.6";
+    this->userAgentString = "Dianara/0.7";
 
     this->postsPerPage = 20;
+    this->updatesToTimelineBlocked = false;
 
     qoauth = new QOAuth::Interface();
     qoauth->setRequestTimeout(10000); // 10 sec timeout
@@ -88,6 +89,16 @@ void PumpController::setPostsPerPage(int ppp)
     qDebug() << "PumpController: setting postsPerPage to" << this->postsPerPage;
 }
 
+/*
+ *
+ *
+ */
+void PumpController::setUpdatesToTimelineBlocked(bool blocked)
+{
+    this->updatesToTimelineBlocked = blocked;
+    qDebug() << "PumpController: Updates to timeline blocked? " << this->updatesToTimelineBlocked;
+}
+
 
 
 /*
@@ -135,7 +146,7 @@ void PumpController::setUserCredentials(QString userID)
     this->getUserProfile(this->userID);
 
 
-    // This will call getContactList() and getTimeLine(), etc.
+    // This will call getContactList() and getMainTimeline(), etc.
     this->initialDataStep = 0;
     this->initialDataTimer->start(2000);  // start 2 seconds after setting the ID
                                           // (mainly on program startup)
@@ -247,13 +258,20 @@ void PumpController::getContactList(QString listType)
 
 
 /*
+ * Get main timeline
+ *
  * GET https://pumpserver.example/api/username/inbox/major
  *
  */
-void PumpController::getTimeline(int timelineOffset)
+void PumpController::getMainTimeline(int timelineOffset)
 {
-    emit currentJobChanged(tr("Getting timeline..."));
-    emit timeLineOffsetChanged(timelineOffset);
+    if (this->updatesToTimelineBlocked)
+    {
+        qDebug() << "Updating timelines requested, but updates are blocked; ignoring";
+        return;
+    }
+
+    emit currentJobChanged(tr("Getting main timeline..."));
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/inbox/major";
 
@@ -262,7 +280,7 @@ void PumpController::getTimeline(int timelineOffset)
     paramMap.insert("offset", QString("%1").arg(timelineOffset).toLocal8Bit());
 
     QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
-                                                           TimelineRequest, paramMap);
+                                                           MainTimelineRequest, paramMap);
 
 
     qDebug() << "===================================";
@@ -277,6 +295,82 @@ void PumpController::getTimeline(int timelineOffset)
 
 
 
+/*
+ * Get direct timeline, posts with the user's address in the "To:" field,
+ * that is, sent explicitly to the user
+ *
+ * GET https://pumpserver.example/api/username/inbox/direct
+ *
+ */
+void PumpController::getDirectTimeline(int timelineOffset)
+{
+    emit currentJobChanged(tr("Getting direct messages timeline..."));
+
+    QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/inbox/direct/major";
+
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", QString("%1").arg(this->postsPerPage).toLocal8Bit());
+    paramMap.insert("offset", QString("%1").arg(timelineOffset).toLocal8Bit());
+
+    QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
+                                                           DirectTimelineRequest, paramMap);
+
+    nam.get(timelineRequest);
+}
+
+
+
+/*
+ * Get activity timeline, user's own posts
+ *
+ * GET https://pumpserver.example/api/username/feed/major
+ *
+ */
+void PumpController::getActivityTimeline(int timelineOffset)
+{
+    emit currentJobChanged(tr("Getting activity timeline..."));
+
+    QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed/major";
+
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", QString("%1").arg(this->postsPerPage).toLocal8Bit());
+    paramMap.insert("offset", QString("%1").arg(timelineOffset).toLocal8Bit());
+
+    QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
+                                                           ActivityTimelineRequest, paramMap);
+
+    nam.get(timelineRequest);
+}
+
+
+
+/*
+ * Get favorites timeline, posts where user clicked "like"
+ *
+ * GET https://pumpserver.example/api/username/favorites
+ *
+ */
+void PumpController::getFavoritesTimeline(int timelineOffset)
+{
+    emit currentJobChanged(tr("Getting favorites timeline..."));
+
+    QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/favorites";
+
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", QString("%1").arg(this->postsPerPage).toLocal8Bit());
+    paramMap.insert("offset", QString("%1").arg(timelineOffset).toLocal8Bit());
+
+    QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
+                                                           FavoritesTimelineRequest, paramMap);
+
+    nam.get(timelineRequest);
+}
+
+
+
+
+
+
 /*
  * Get list of people who liked a specific post
  *
@@ -334,6 +428,30 @@ void PumpController::getPostShares(QString postSharesURL)
 
 
 
+/*
+ * Get the minor feed, used in the "Meanwhile" column
+ *
+ * GET https://pumpserver.example/api/username/inbox/minor
+ *
+ */
+void PumpController::getMinorFeed()
+{
+    emit currentJobChanged(tr("Getting minor feed..."));
+
+    QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/inbox/minor";
+
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", "40");
+
+    QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
+                                                           MinorFeedRequest, paramMap);
+
+
+    nam.get(timelineRequest);
+}
+
+
+
 /*
  * Prepare a QNetworkRequest with OAuth header, content type and user agent.
  *
@@ -381,11 +499,17 @@ void PumpController::uploadFile(QString filename, QString contentType)
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/uploads";
 
+    /*QOAuth::ParamMap paramMap;
+    paramMap.insert("title", "imgTitle");
+    paramMap.insert("description", "imgDescription");*/
     QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST,
                                                        UploadFileRequest,
                                                        QOAuth::ParamMap(),
                                                        contentType);
 
+    /*url.append("?title=imgTitle&description=imgDescription");
+    postRequest.setUrl(QUrl(url));*/
+
     QFile file(filename);
     file.open(QIODevice::ReadOnly);
 
@@ -452,57 +576,28 @@ void PumpController::requestFinished(QNetworkReply *reply)
         emit showNotification("Bad Request (400)\n" + reply->url().toString());
         qDebug() << "HTTP 400: Bad Request.";
         qDebug() << "Data:  " << reply->readAll();
-        //qDebug() << reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
         return;
 
     case 0:
         emit currentJobChanged(tr("Error connecting to %1").arg(reply->url().host()) + ": " + reply->errorString());
         return;
 
+    // Other HTTP codes
+    default:
+        qDebug() << "Unhandled HTTP error " << httpCode;
+        emit showNotification(tr("Unhandled HTTP error code %1").arg(httpCode)
+                              + "\n" + reply->url().toString());
+
+        return;
+
+
+    //////////////////////////////////////// The good one!
     case 200:
         qDebug() << "HTTP 200: OK!";
     }
 
 
-    if (httpCode == 200) // If OK received
-    {
-        switch (requestType) // Inform in status bar which operation was successful
-        {
-        case PublishPostRequest:
-            emit currentJobChanged(tr("Post published successfully."));
-            break;
-
-        case LikePostRequest:
-            emit currentJobChanged(tr("Post liked successfully."));
-            break;
-
-        case CommentPostRequest:
-            emit currentJobChanged(tr("Comment posted successfully."));
-            break;
-
-        case SharePostRequest:
-            emit currentJobChanged(tr("Post shared successfully."));
-            break;
-
-        case DeletePostRequest:
-            emit currentJobChanged(tr("Post deleted successfully."));
-            break;
-
-        case FollowContactRequest:
-            emit currentJobChanged(tr("Following successfully."));
-
-            // Re-request the contact list, now with a new contact (not very optimized...)
-            this->getContactList("following");
-            break;
-
-        case UnfollowContactRequest:
-            emit currentJobChanged(tr("Stopped following successfully."));
-
-            // Re-request the contact list, now with one contact less (not very optimized...)
-            this->getContactList("following");
-            break;
-        }
-    }
+    // At this point, httpCode should be 200 = OK
 
 
     // Read all received data
@@ -515,14 +610,22 @@ void PumpController::requestFinished(QNetworkReply *reply)
     QVariantList jsonDataList;
 
 
+    // Unless it was an AvatarRequest or an ImageRequest, it should be JSON, so parse it
+    if (requestType != AvatarRequest && requestType != ImageRequest)
+    {
+        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
+        qDebug() << "JSON data size (items):" << jsonData.size();
+        qDebug() << "Keys:" << jsonData.keys();
+    }
+
+
     switch (requestType)
     {
 
     case ClientRegistrationRequest:
         qDebug() << "Client Registration was requested";
 
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << jsonData;
+        qDebug() << "Raw JSON:" << jsonData;
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
@@ -536,11 +639,9 @@ void PumpController::requestFinished(QNetworkReply *reply)
             settings.setValue("clientID",     this->clientID);
             settings.setValue("clientSecret", this->clientSecret);
 
-
             this->getToken();
         }
 
-
         break;
 
 
@@ -548,9 +649,6 @@ void PumpController::requestFinished(QNetworkReply *reply)
     case UserProfileRequest:
         qDebug() << "A user profile was requested";
 
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << jsonData;
-
         if (jsonParsedOK && jsonData.size() > 0)
         {
             QString profId = jsonData["profile"].toMap()["id"].toString();
@@ -573,12 +671,15 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
 
-    ////////////////////////////////////////////////// If a timeline was requested
-    case TimelineRequest:
-        qDebug() << "A Timeline was requested";
-
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << "JSON data size (items):" << jsonData.size();
+    ////////////////////////////////////////////////// If an timeline was requested
+    case MainTimelineRequest:
+        // just jump to the next
+    case DirectTimelineRequest:
+        // just jump to next
+    case ActivityTimelineRequest:
+        // just... yeah, jump
+    case FavoritesTimelineRequest:
+        qDebug() << "A timeline was requested";
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
@@ -586,12 +687,36 @@ void PumpController::requestFinished(QNetworkReply *reply)
             emit currentJobChanged(tr("Timeline received. Updating post list..."));
 
             jsonDataList = jsonData["items"].toList();
-            qDebug() << "Number of items in inbox timeline:" << jsonDataList.size();
+            qDebug() << "Number of items in timeline:" << jsonDataList.size();
 
             QString previousLink = jsonData["links"].toMap().value("prev").toMap().value("href").toString();
             QString nextLink = jsonData["links"].toMap().value("next").toMap().value("href").toString();
 
-            emit timeLineReceived(jsonDataList, this->postsPerPage, previousLink, nextLink);
+            if (requestType == MainTimelineRequest)
+            {
+                qDebug() << "It was the main timeline";
+                emit mainTimelineReceived(jsonDataList, this->postsPerPage,
+                                          previousLink, nextLink);
+            }
+            else if (requestType == DirectTimelineRequest)
+            {
+                qDebug() << "It was the direct messages timeline";
+                emit directTimelineReceived(jsonDataList, this->postsPerPage,
+                                            previousLink, nextLink);
+            }
+            else if (requestType == ActivityTimelineRequest)
+            {
+                qDebug() << "It was the own activity timeline";
+                emit activityTimelineReceived(jsonDataList, this->postsPerPage,
+                                              previousLink, nextLink);
+            }
+            else if (requestType == FavoritesTimelineRequest)
+            {
+                qDebug() << "It was the favorites timeline";
+                emit favoritesTimelineReceived(jsonDataList, this->postsPerPage,
+                                               previousLink, nextLink);
+                qDebug() << "#####################";
+            }
         }
         else
         {
@@ -601,13 +726,26 @@ void PumpController::requestFinished(QNetworkReply *reply)
         break;
 
 
+
+    ////////////////////////////////////
+    case PublishPostRequest:
+        emit currentJobChanged(tr("Post published successfully."));
+        emit postPublished();
+        break;
+
+
+
+    ///////////////////////////////////// If liking a post was requested
+    case LikePostRequest:
+        emit currentJobChanged(tr("Post liked successfully."));
+        emit likeSet();
+        break;
+
+
     ///////////////////////////////////// If the likes for a post were requested
     case PostLikesRequest:
         qDebug() << "Likes for a post were requested" << reply->url().toString();
 
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << "JSON data size (items):" << jsonData.size();
-
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
@@ -627,13 +765,19 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
 
+    ///////////////////////////////////// If commenting on a post was requested
+    case CommentPostRequest:
+        emit currentJobChanged(tr("Comment posted successfully."));
+        emit commentPosted(); // This will be catched by Commenter()
+
+        break;
+
+
+
     ///////////////////////////////////// If the comments for a post were requested
     case PostCommentsRequest:
         qDebug() << "Comments for a post were requested" << reply->url().toString();
 
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << "JSON data size (items):" << jsonData.size();
-
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
@@ -653,19 +797,133 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
 
-    case PostSharesRequest:
+    case SharePostRequest:
+        emit currentJobChanged(tr("Post shared successfully."));
         break;
 
+    case PostSharesRequest:
+        // TODO
+        break;
+
+
+
+    case MinorFeedRequest:
+        qDebug() << "The minor feed was requested";
+
+        if (jsonParsedOK && jsonData.size() > 0)
+        {
+            qDebug() << "JSON parsed OK";
+            emit currentJobChanged(tr("Minor feed received."));
+
+            jsonDataList = jsonData["items"].toList();
+            qDebug() << "Number of items in mitnor feed:" << jsonDataList.size();
+
+             emit minorFeedReceived(jsonDataList);
+        }
+        else
+        {
+            qDebug() << "Error parsing received JSON data!";
+            qDebug() << "Raw data:" << packetData; // JSON directly
+        }
+        break;
+
+
+
+    case DeletePostRequest:
+        emit currentJobChanged(tr("Post deleted successfully."));
+        break;
+
+
+    case FollowContactRequest:
+        emit currentJobChanged(tr("Following successfully."));
+
+        // Re-request the contact list, now with a new contact (not very optimized...)
+        this->getContactList("following");
+
+        break;
+
+    case UnfollowContactRequest:
+        emit currentJobChanged(tr("Stopped following successfully."));
+
+        // Re-request the contact list, now with one contact less (not very optimized...)
+        this->getContactList("following");
+
+        break;
+
+
+    case FollowingListRequest:
+        // just go to the next
+    case FollowersListRequest:
+        qDebug() << "A contact list was requested";
+
+        if (jsonParsedOK && jsonData.size() > 0)
+        {
+            qDebug() << "JSON parsed OK";
+
+            QVariant contactsVariant = jsonData.value("items");
+            if (contactsVariant.type() == QVariant::List)
+            {
+                qDebug() << "Parsed a List, listing contacts...";
+
+                if (requestType == FollowingListRequest)
+                {
+                    emit currentJobChanged(tr("List of 'following' received."));
+                    emit contactListReceived("following", contactsVariant.toList());
+                }
+                else // == FollowersListRequest
+                {
+                    emit currentJobChanged(tr("List of 'followers' received."));
+                    emit contactListReceived("followers", contactsVariant.toList());
+                }
+            }
+            else
+            {
+                qDebug() << "Expected a list of contacts, received something else:";
+                qDebug() << jsonData;
+            }
+        }
+        else
+        {
+            qDebug() << "Error parsing received JSON data!";
+            qDebug() << "Raw data:" << packetData; // JSON directly
+        }
+
+        break;
+
+
+
+    case AvatarRequest:
+        qDebug() << "Received AVATAR data, from " << reply->url();
+        if (reply->isFinished())
+        {
+            qDebug() << "Avatar received 100%";
+            emit avatarPictureReceived(packetData, reply->url());
+        }
+        else
+        {
+            qDebug() << "Avatar not complete yet";
+        }
+        break;
+
+
+    case ImageRequest:
+        qDebug() << "Received IMAGE data, from " << reply->url();
+        if (reply->isFinished())
+        {
+            qDebug() << "Image received 100%";
+            emit imageReceived(packetData, reply->url());
+        }
+        else
+        {
+            qDebug() << "Image not complete yet";
+        }
+        break;
 
 
     //////////////////////////////////////// If uploading a file was requested
     case UploadFileRequest:
         qDebug() << "Uploading a file was requested";
 
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << "JSON data size (items):" << jsonData.size();
-        qDebug() << jsonData.keys();
-
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
@@ -689,85 +947,6 @@ void PumpController::requestFinished(QNetworkReply *reply)
         break;
 
 
-
-
-
-
-    case FollowingListRequest:
-        // just go to next
-    case FollowersListRequest:
-        qDebug() << "A contact list was requested";
-
-        jsonData = jsonParser.parse(packetData, &jsonParsedOK).toMap();
-        qDebug() << "JSON data size (items):" << jsonData.size();
-
-        if (jsonParsedOK && jsonData.size() > 0)
-        {
-            qDebug() << "JSON parsed OK";
-
-            QVariant contactsVariant = jsonData.value("items");
-            if (contactsVariant.type() == QVariant::List)
-            {
-                qDebug() << "Parsed a List, listing contacts...";
-
-                if (requestType == FollowingListRequest)
-                {
-                    emit currentJobChanged(tr("Followed list received."));
-                    emit contactListReceived("following", contactsVariant.toList());
-                }
-                else // == FollowersListRequest
-                {
-                    emit currentJobChanged(tr("Followers list received."));
-                    emit contactListReceived("followers", contactsVariant.toList());
-                }
-            }
-            else
-            {
-                qDebug() << "Expected a list of contacts, received something else:";
-                qDebug() << jsonData;
-            }
-        }
-        else
-        {
-            qDebug() << "Error parsing received JSON data!";
-            qDebug() << "Raw data:" << packetData; // JSON directly
-        }
-
-        break;
-
-
-
-    case AvatarRequest:
-        qDebug() << "Received AVATAR data, from " << reply->url();
-        // FIXME: take care of possible redirections
-
-        if (reply->isFinished())
-        {
-            qDebug() << "Avatar received 100%";
-            emit avatarPictureReceived(packetData, reply->url());
-        }
-        else
-        {
-            qDebug() << "Avatar not complete yet";
-        }
-        break;
-
-
-    case ImageRequest:
-        qDebug() << "Received IMAGE data, from " << reply->url();
-
-        if (reply->isFinished())
-        {
-            qDebug() << "Image received 100%";
-            emit imageReceived(packetData, reply->url());
-        }
-        else
-        {
-            qDebug() << "Image not complete yet";
-        }
-        break;
-
-
     }
     // end switch (requestType)
 }
@@ -775,12 +954,31 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
 
+
+
 void PumpController::sslErrorsHandler(QNetworkReply *reply, QList<QSslError> errorList)
 {
-    qDebug() << "\n\n==== SSL errors!! ====";
+    qDebug() << "\n==== SSL errors!! ====";
     qDebug() << "At:" << reply->url();
     qDebug() << "Error list:" << errorList << "\n\n";
 
+    // ignore completely for now
+    qDebug() << "Ignoring these errors...";
+    reply->ignoreSslErrors();
+
+    /*
+    // Check list of SSL errors
+    foreach (QSslError sslError, errorList)
+    {
+        // Ignore the "certificate is self-signed" SSL error, it's expected
+        if (sslError.error() == QSslError::SelfSignedCertificate)
+        {
+            qDebug() << "Ignoring self-signed certificate error";
+            reply->ignoreSslErrors(); // FIXME, maybe ask the user
+            return;
+        }
+    }
+    */
 }
 
 
@@ -931,25 +1129,45 @@ void PumpController::getInitialData()
     switch (this->initialDataStep)
     {
     case 0:
-        this->getTimeline(0);
+        this->getMainTimeline(0);
         break;
 
     case 1:
-        this->getContactList("following");
+        this->getDirectTimeline(0);
         break;
 
     case 2:
+        this->getActivityTimeline(0);
+        break;
+
+    case 3:
+        this->getFavoritesTimeline(0);
+        break;
+
+    case 4:
+        this->getMinorFeed();
+        break;
+
+    case 5:
+        this->getContactList("following");
+        break;
+
+    case 6:
         this->getContactList("followers");
         break;
 
+    case 7:
+        // Do nothing, so it takes longer for the final (default) step to arrive
+        break;
 
 
     default:
-        emit currentJobChanged(tr("Ready.")); // FIXME
+        emit currentJobChanged(tr("Ready."));
         initialDataTimer->stop();
 
-        qDebug() << "All initial data loaded ------";
-        qDebug() << "------------------------------";
+        qDebug() << "--------------------------------------";
+        qDebug() << "-- All initial data loaded -----------";
+        qDebug() << "--------------------------------------";
     }
 
     ++initialDataStep;
@@ -1269,11 +1487,3 @@ void PumpController::unfollowContact(QString address)
 
     nam.post(unfollowRequest, data);
 }
-
-
-
-
-void PumpController::setCurrentTimeline(QString link)
-{
-    qDebug() << "PumpController::setCurrentTimeline()" << link;
-}
