@@ -60,6 +60,7 @@ public:
     ~PumpController();
 
     void setPostsPerPage(int ppp);
+    void setUpdatesToTimelineBlocked(bool blocked);
 
     void setNewUserID(QString userID);
     void setUserCredentials(QString userID);
@@ -70,12 +71,17 @@ public:
     void getAvatar(QString avatarURL);
     void getImage(QString imageURL);
     void getContactList(QString listType);
-    void getTimeline(int timelineOffset);
+
+    void getMainTimeline(int timelineOffset);
+    void getDirectTimeline(int timelineOffset);
+    void getActivityTimeline(int timelineOffset);
+    void getFavoritesTimeline(int timelineOffset);
 
     void getPostLikes(QString postLikesURL);
     void getPostComments(QString postCommentsURL);
     void getPostShares(QString postSharesURL);
 
+    void getMinorFeed();
 
     QNetworkRequest prepareRequest(QString url,
                                    QOAuth::HttpMethod method,
@@ -96,13 +102,16 @@ public:
         FollowingListRequest,
         FollowersListRequest,
 
-        TimelineRequest,
+        MainTimelineRequest,
+        DirectTimelineRequest,
+        ActivityTimelineRequest,
+        FavoritesTimelineRequest,
+
         PostLikesRequest,
         PostCommentsRequest,
         PostSharesRequest,
 
-        AvatarRequest,
-        ImageRequest,
+        MinorFeedRequest,
 
         PublishPostRequest,
         LikePostRequest,
@@ -113,6 +122,10 @@ public:
         FollowContactRequest,
         UnfollowContactRequest,
 
+
+        AvatarRequest,
+        ImageRequest,
+
         UploadFileRequest
     };
 
@@ -124,15 +137,28 @@ signals:
 
     void profileReceived(QString fullName, QString avatarURL);
     void contactListReceived(QString listType, QVariantList contactList);
-    void timeLineReceived(QVariantList postList, int postsPerPage,
-                          QString previousLink, QString nextLink);
+
+    void mainTimelineReceived(QVariantList postList, int postsPerPage,
+                              QString previousLink, QString nextLink);
+    void directTimelineReceived(QVariantList postList, int postsPerPage,
+                                QString previousLink, QString nextLink);
+    void activityTimelineReceived(QVariantList postList, int postsPerPage,
+                                  QString previousLink, QString nextLink);
+    void favoritesTimelineReceived(QVariantList postList, int postsPerPage,
+                                   QString previousLink, QString nextLink);
+
     void likesReceived(QVariantList likesList, QString originatingPostURL);
     void commentsReceived(QVariantList commentsList, QString originatingPostURL);
 
-    void timeLineOffsetChanged(int offset);
+    void minorFeedReceived(QVariantList activitiesList);
+
     void avatarPictureReceived(QByteArray pictureData, QUrl pictureURL);
     void imageReceived(QByteArray pictureData, QUrl pictureURL);
 
+    void postPublished();
+    void likeSet();
+    void commentPosted();
+
     void showNotification(QString message);
     void currentJobChanged(QString message);
 
@@ -140,7 +166,8 @@ signals:
 
 public slots:
     void requestFinished(QNetworkReply *reply);
-    void sslErrorsHandler(QNetworkReply *reply, QList<QSslError> errorList); // Tests
+
+    void sslErrorsHandler(QNetworkReply *reply, QList<QSslError> errorList);
 
     void getToken();
     void authorizeApplication(QString verifierCode);
@@ -162,8 +189,6 @@ public slots:
     void followContact(QString address);
     void unfollowContact(QString address);
 
-    void setCurrentTimeline(QString link);
-
 
 
 private:
@@ -193,6 +218,8 @@ private:
 
 
     int postsPerPage;
+    bool updatesToTimelineBlocked;
+
 
     QString imageTitle;
     QString imageDescription;
