@@ -20,27 +20,78 @@
 
 #include "timeline.h"
 
-TimeLine::TimeLine(PumpController *pumpController, QWidget *parent) :  QWidget(parent)
+TimeLine::TimeLine(int timelineType,
+                   PumpController *pumpController,
+                   QWidget *parent) :  QWidget(parent)
 {
+    this->timelineType = timelineType;
+
     this->pController = pumpController;
-    connect(pController, SIGNAL(timeLineOffsetChanged(int)),
-            this, SLOT(syncTimelineOffset(int)));  // keep timeline offset coherent
 
     mainLayout = new QVBoxLayout();
     mainLayout->setAlignment(Qt::AlignTop);
 
 
     QMap<QString,QString> demoPostData;
-    demoPostData.insert("author", "DemoUser");
-    demoPostData.insert("text",   "<h2><b>Demo Post</b></h2><br>"
-                                  "Press <b>F5</b> or <i>Update timelines</i> in the menu.<br><br>"
-                                  "Upon startup, timeline should update automatically.<br><br>"
-                                  "<a href=\"http://jancoding.wordpress.com/dianara\">Dianara's blog</a><br>"
-                                  "<a href=\"http://dianara.nongnu.org\">Dianara's website</a><br><br>");
-    demoPostData.insert("createdAt", "2000-01-01T00:01:02Z");
-    demoPostData.insert("likesCount", "32");
-    demoPostData.insert("commentsCount", "128");
-    demoPostData.insert("resharesCount", "256");
+    demoPostData.insert("author",         "Demo User");
+    demoPostData.insert("authorId",       "demo@somepump.example");
+    demoPostData.insert("authorBio",      "I am not a real user");
+    demoPostData.insert("authorHometown", "Demoville");
+    demoPostData.insert("generator",      "Dianara");
+
+
+    // Demo post content depends on timeline type
+    switch (this->timelineType)
+    {
+    case TimelineTypeMain:
+        demoPostData.insert("text",   "<h2><b>Welcome to Dianara</b></h2><br>"
+
+                                      "Dianara is a <a href=\"http://pump.io\">pump.io</a> client.<br><br>"
+
+                                      "After you have set up your account in <b>Settings > Account Details</b>,<br>"
+                                      "you can press <b>F5</b> or <i>Update timelines</i> in the menu.<br><br>"
+
+                                      "Upon startup, timeline should update automatically.<br><br><br>"
+
+                                      "<a href=\"http://jancoding.wordpress.com/dianara\">Dianara's blog</a><br>"
+                                      "<br>");
+        break;
+
+
+    case TimelineTypeDirect:
+        demoPostData.insert("text",   "<h2><b>Direct Messages Timeline</b></h2><br>"
+
+                                      "Here, you'll see posts specifically directed to you."
+
+                                      "<br>");
+        break;
+
+
+    case TimelineTypeActivity:
+        demoPostData.insert("text",   "<h2><b>Activity Timeline</b></h2><br>"
+
+                                      "You'll see your own posts here."
+
+                                      "<br>");
+        break;
+
+
+    case TimelineTypeFavorites:
+        demoPostData.insert("text",   "<h2><b>Favorites Timeline</b></h2><br>"
+
+                                      "Posts you've liked."
+
+                                      "<br>");
+        break;
+
+
+
+    default:
+        demoPostData.insert("text", "<h2>Empty timeline</h2>");
+
+    }
+
+    demoPostData.insert("createdAt", "2012-01-09T00:00:00Z");
 
 
     splitter = new QSplitter(Qt::Vertical, this);
@@ -53,6 +104,12 @@ TimeLine::TimeLine(PumpController *pumpController, QWidget *parent) :  QWidget(p
     firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), tr("&First Page"));
     connect(firstPageButton, SIGNAL(clicked()),
             this, SLOT(goToFirstPage()));
+
+    currentPageLabel = new QLabel();
+    currentPageLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
+    currentPageLabel->setSizePolicy(QSizePolicy::Maximum,
+                                    QSizePolicy::Maximum);
+
     previousPageButton = new QPushButton(QIcon::fromTheme("go-previous"), tr("&Previous Page"));
     connect(previousPageButton, SIGNAL(clicked()),
             this, SLOT(goToPreviousPage()));
@@ -60,20 +117,27 @@ TimeLine::TimeLine(PumpController *pumpController, QWidget *parent) :  QWidget(p
     connect(nextPageButton, SIGNAL(clicked()),
             this, SLOT(goToNextPage()));
 
+
     bottomLayout = new QHBoxLayout();
     bottomLayout->addWidget(firstPageButton);
-    bottomLayout->addSpacing(16);
+    bottomLayout->addSpacing(12);
+    bottomLayout->addWidget(currentPageLabel);
+    bottomLayout->addSpacing(12);
     bottomLayout->addWidget(previousPageButton);
     bottomLayout->addWidget(nextPageButton);
     mainLayout->addLayout(bottomLayout);
 
+
     // Add the default "demo" post
-    splitter->addWidget(new Post(pController, demoPostData, QVariantList(), this));
+    splitter->addWidget(new Post(pController, demoPostData,
+                                 QVariantList(), QVariantList(), QVariantList(),
+                                 this));
 
 
     this->setLayout(mainLayout);
 
 
+    // FIXME: take care of this for the different timelines
     QSettings settings;
     this->previousNewestPostID = settings.value("previousNewestPostID", "").toString();
 
@@ -118,7 +182,7 @@ void TimeLine::clearTimeLineContents()
     this->postsInTimeline.clear();
 
     /*
-     * This is to avoid the splitter getting bigger and bigger everytime the user
+     * This is to avoid the splitter getting bigger and bigger every time the user
      * updates the timeline (F5, etc)
      *
      */
@@ -128,6 +192,44 @@ void TimeLine::clearTimeLineContents()
 
 
 
+void TimeLine::requestTimelinePage()
+{
+    switch (this->timelineType)
+    {
+    case TimelineTypeMain:
+        this->pController->getMainTimeline(this->timelineOffset);
+        break;
+
+    case TimelineTypeDirect:
+        this->pController->getDirectTimeline(this->timelineOffset);
+        break;
+
+    case TimelineTypeActivity:
+        this->pController->getActivityTimeline(this->timelineOffset);
+        break;
+
+    case TimelineTypeFavorites:
+        this->pController->getFavoritesTimeline(this->timelineOffset);
+        break;
+    }
+
+}
+
+
+/*
+ *  Update the label at the bottom of the page, indicating current "page"
+ *
+ */
+void TimeLine::updateCurrentPageNumber()
+{
+    QString currentPageString;
+    currentPageString = QString("%1").arg((this->timelineOffset / this->postsPerPage) + 1);
+
+    this->currentPageLabel->setText(currentPageString);
+}
+
+
+
 /*********************************************************/
 /*********************************************************/
 /************************ SLOTS **************************/
@@ -162,10 +264,21 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             this->nextPageLink = nextLink;
 
             QVariantMap singlePostMap = singlePost.toMap();
+            QVariantMap objectMap;
+            if (this->timelineType != TimelineTypeFavorites)  // VERY TMP!! Favorites-tmp-stuff
+            {
+                objectMap = singlePostMap["object"].toMap();
+            }
+            else
+            {
+                // Since "favorites" is a collection of objects directly, not activities
+                objectMap = singlePostMap;
+            }
 
-            //QString itemVerb = singlePostMap.value("verb").toString();
 
-            QString postIsDeleted = singlePostMap.value("object").toMap().value("deleted").toString();
+            QString itemVerb = singlePostMap["verb"].toString();
+
+            QString postIsDeleted = objectMap["deleted"].toString();
 
 
             QString postID;
@@ -177,21 +290,23 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             QString postAuthorHometown;
             QString postAuthorBio;
             QString postGenerator;
+            QString postCreatedAt;
+            QString postTo;
+            QString postCC;
             QString postTitle;
             QString postImage;
             QString postText;
             QString postIsNsfw;
-            QString postCreatedAt;
-            QString postRootAuthor;
             QString postLikesCount;
             QString postCommentsCount;
             QString postResharesCount;
 
             QString postIsLiked;
             QString postIsShared;
+            QString postSharedBy;
 
 
-            postID = singlePostMap.value("object").toMap().value("id").toString();
+            postID = objectMap["id"].toString();
             if (newestPostID.isEmpty()) // only first time, for newest post
             {
                 if (this->timelineOffset == 0)
@@ -217,24 +332,85 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 }
             }
 
-            postURL = singlePostMap.value("object").toMap().value("url").toString();
+            postURL = objectMap["url"].toString();
 
-            postType = singlePostMap.value("object").toMap().value("objectType").toString();
+            postType = objectMap["objectType"].toString();
 
-            postIsLiked = singlePostMap.value("object").toMap().value("liked").toString();
-            postIsShared = singlePostMap.value("object").toMap().value("pump_io").toMap().value("shared").toString();
+            postIsLiked = objectMap["liked"].toString();
 
-            postAuthor = singlePostMap.value("actor").toMap().value("displayName").toString();
-            postAuthorAvatar = singlePostMap.value("actor").toMap().value("image").toMap().value("url").toString();
-            // for Author ID, remove the first 5 characters from the field, "acct:"
-            postAuthorId = singlePostMap.value("actor").toMap().value("id").toString().remove(0,5);
+            QVariantMap authorMap;
+            if (itemVerb == "share")
+            {
+                postIsShared = "true";
 
-            postAuthorBio = singlePostMap.value("actor").toMap().value("summary").toString();
-            postAuthorHometown = singlePostMap.value("actor").toMap().value("location").toMap().value("displayName").toString();
+                // Get original author data
+                authorMap = objectMap["author"].toMap();
+
+                postSharedBy = singlePostMap["actor"].toMap().value("displayName").toString();
+            }
+            else
+            {
+                postIsShared = "false";
+
+                // Not a share, so no original author, just get the actor
+                authorMap = singlePostMap["actor"].toMap();
+            }
+
+            //// VERY TMP!! Favorites-tmp-stuff
+            if (this->timelineType==TimelineTypeFavorites)
+            {
+                authorMap = objectMap["author"].toMap();
+            }
+
+            postAuthor = authorMap["displayName"].toString();
+            postAuthorAvatar = authorMap["image"].toMap().value("url").toString();
+            postAuthorId = authorMap["id"].toString();
+            // For Author ID, remove the first 5 characters from the field, if they are "acct:"
+            // (users have acct: in from of the ID, but services like OFG don't)
+            if (postAuthorId.startsWith("acct:"))
+            {
+                postAuthorId.remove(0,5);
+            }
+
+            postAuthorBio = authorMap["summary"].toString();
+            postAuthorHometown = authorMap["location"].toMap().value("displayName").toString();
+
+
+            postCreatedAt = objectMap["published"].toString();
+            postGenerator = singlePostMap["generator"].toMap().value("displayName").toString();
+
+
+            // Audience
+            foreach (QVariant postToVariant, singlePostMap.value("to").toList())
+            {
+                if (postToVariant.toMap().value("id").toString()
+                  == "http://activityschema.org/collection/public")
+                {
+                    postTo += tr("Public") + ", ";
+                }
+                else
+                {
+                    postTo += postToVariant.toMap().value("displayName").toString() + ", ";
+                }
+            }
+            postTo.remove(-2, 2); // remove last comma and space
+
+            // CC
+            foreach (QVariant postCCVariant, singlePostMap.value("cc").toList())
+            {
+                if (postCCVariant.toMap().value("id").toString()
+                  == "http://activityschema.org/collection/public")
+                {
+                    postCC += tr("Public") + ", ";
+                }
+                else
+                {
+                    postCC += postCCVariant.toMap().value("displayName").toString() + ", ";
+                }
+            }
+            postCC.remove(-2, 2);
 
-            postGenerator = singlePostMap.value("generator").toMap().value("displayName").toString();
 
-            postCreatedAt = singlePostMap.value("object").toMap().value("published").toString();
 
 
             ///
@@ -243,60 +419,61 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
             if (postType == "image")
             {
-                postTitle = singlePostMap.value("object").toMap().value("displayName").toString();
+                postTitle = objectMap["displayName"].toString();
 
 
                 // See if there's a proxy URL for the image first (for private images)
-                postImage = singlePostMap.value("object").toMap().value("fullImage").toMap().value("pump_io").toMap().value("proxyURL").toString();
+                postImage = objectMap["fullImage"].toMap().value("pump_io").toMap().value("proxyURL").toString();
                 qDebug() << "Trying Proxyed fullImage";
 
                 // if that's empty, try the "small" version
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying Proxyed thumbnail image";
-                    postImage = singlePostMap.value("object").toMap().value("image").toMap().value("pump_io").toMap().value("proxyURL").toString();
+                    postImage = objectMap["image"].toMap().value("pump_io").toMap().value("proxyURL").toString();
                 }
 
                 // If that's also empty, use regular fullImage->url field
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying direct fullImage";
-                    postImage = singlePostMap.value("object").toMap().value("fullImage").toMap().value("url").toString();
+                    postImage = objectMap["fullImage"].toMap().value("url").toString();
                 }
 
                 // And if that is ALSO empty, use regular image->url
                 if (postImage.isEmpty())
                 {
                     qDebug() << "Trying direct thumbnail image";
-                    postImage = singlePostMap.value("object").toMap().value("image").toMap().value("url").toString();
+                    postImage = objectMap["image"].toMap().value("url").toString();
                 }
                 qDebug() << "postImage:" << postImage;
             }
 
 
-            postText = singlePostMap.value("object").toMap().value("content").toString();
+            postText = objectMap["content"].toString();
 
-            postLikesCount = singlePostMap.value("object").toMap().value("likes").toMap().value("totalItems").toString();
-            postCommentsCount = singlePostMap.value("object").toMap().value("replies").toMap().value("totalItems").toString();
-            postResharesCount = singlePostMap.value("object").toMap().value("shares").toMap().value("totalItems").toString();
-
-            // get comments list here, pass to the Post()
-            QVariantList lastCommentsList = singlePostMap.value("object").toMap().value("replies").toMap().value("items").toList();
+            postLikesCount = objectMap["likes"].toMap().value("totalItems").toString();
+            postCommentsCount = objectMap["replies"].toMap().value("totalItems").toString();
+            postResharesCount = objectMap["shares"].toMap().value("totalItems").toString();
 
+            // get last likes, comments and shares list here, pass to the Post()
+            QVariantList lastLikesList = objectMap["likes"].toMap().value("items").toList();
+            QVariantList lastCommentsList = objectMap["replies"].toMap().value("items").toList();
+            QVariantList lastSharesList = objectMap["shares"].toMap().value("items").toList();
 
             // Get URL for likes; first, proxyURL if it exists
-            QString postLikesURL = singlePostMap.value("object").toMap().value("likes").toMap().value("pump_io").toMap().value("proxyURL").toString();
+            QString postLikesURL = objectMap["likes"].toMap().value("pump_io").toMap().value("proxyURL").toString();
             // If still empty, get regular URL (that means the post is in the same server we are)
             if (postLikesURL.isEmpty())
             {
-                postLikesURL = singlePostMap.value("object").toMap().value("likes").toMap().value("url").toString();
+                postLikesURL = objectMap["likes"].toMap().value("url").toString();
             }
 
             // Get URL for comments; first, proxyURL if it exists
-            QString postCommentsURL = singlePostMap.value("object").toMap().value("replies").toMap().value("pump_io").toMap().value("proxyURL").toString();
+            QString postCommentsURL = objectMap["replies"].toMap().value("pump_io").toMap().value("proxyURL").toString();
             if (postCommentsURL.isEmpty()) // If still empty, get regular URL
             {
-                postCommentsURL = singlePostMap.value("object").toMap().value("replies").toMap().value("url").toString();
+                postCommentsURL = objectMap["replies"].toMap().value("url").toString();
             }
 
 
@@ -311,12 +488,15 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             postData.insert("authorBio",          postAuthorBio);
             postData.insert("authorHometown",     postAuthorHometown);
             postData.insert("generator",          postGenerator);
+            postData.insert("createdAt",          postCreatedAt);
+
+            postData.insert("to",                 postTo);
+            postData.insert("cc",                 postCC);
+
             postData.insert("title",              postTitle);
             postData.insert("image",              postImage);
             postData.insert("text",               postText);
-            postData.insert("createdAt",          postCreatedAt);
             postData.insert("nsfw",               postIsNsfw);
-            postData.insert("rootAuthor",         postRootAuthor);
             postData.insert("likesCount",         postLikesCount);
             postData.insert("likesURL",           postLikesURL);
             postData.insert("commentsCount",      postCommentsCount);
@@ -325,12 +505,18 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
             postData.insert("postIsLiked",        postIsLiked);
             postData.insert("postIsShared",       postIsShared);
+            postData.insert("sharedBy",           postSharedBy);
 
 
 
             if (postIsDeleted.isEmpty())  // if post was NOT deleted
             {
-                Post *newPost = new Post(pController, postData, lastCommentsList, this);
+                Post *newPost = new Post(pController,
+                                         postData,
+                                         lastLikesList,
+                                         lastCommentsList,
+                                         lastSharesList,
+                                         this);
                 this->postsInTimeline.append(newPost);
 
                 this->splitter->addWidget(newPost);
@@ -363,6 +549,8 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
     qDebug() << "New posts:" << newPostCount << "; Newest post ID:" << previousNewestPostID;
 
+    this->updateCurrentPageNumber();
+
     emit timelineRendered(newPostCount);
 
     qDebug() << "setTimeLineContents() /END";
@@ -371,17 +559,6 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
 
 
-/*
- * Keep timeline offset in sync
- *
- */
-void TimeLine::syncTimelineOffset(int offset)
-{
-    this->timelineOffset = offset;
-}
-
-
-
 
 /*
  * Add the full list of likes to a post
@@ -399,7 +576,7 @@ void TimeLine::setLikesInPost(QVariantList likesList, QString originatingPostURL
     qDebug() << "Looking for the originating Post() object";
     foreach (Post *post, postsInTimeline)
     {
-        qDebug() << "Checking if" << post->commentsURL() << "==" << originatingPostCleanURL;
+        qDebug() << "Checking if" << post->likesURL() << "==" << originatingPostCleanURL;
 
         if (post->likesURL() == originatingPostCleanURL)
         {
@@ -447,8 +624,10 @@ void TimeLine::setCommentsInPost(QVariantList commentsList, QString originatingP
 void TimeLine::goToFirstPage()
 {
     qDebug() << "TimeLine::goToFirstPage()";
-    this->pController->getTimeline(0); // getTimeline will emit timelineOffsetChanged
-                                       // to keep it sync'd
+
+    this->timelineOffset = 0;
+
+    this->requestTimelinePage();
 }
 
 
@@ -462,7 +641,8 @@ void TimeLine::goToPreviousPage()
     {
         timelineOffset = 0;
     }
-    this->pController->getTimeline(this->timelineOffset);
+
+    this->requestTimelinePage();
 }
 
 
@@ -473,5 +653,5 @@ void TimeLine::goToNextPage()
 
     this->timelineOffset += this->postsPerPage;
 
-    this->pController->getTimeline(this->timelineOffset);
+    this->requestTimelinePage();
 }
