@@ -40,12 +40,27 @@ class TimeLine : public QWidget
     Q_OBJECT
 
 public:
-    TimeLine(PumpController *pumpController,
+    TimeLine(int timelineType,
+             PumpController *pumpController,
              QWidget *parent = 0);
     ~TimeLine();
 
     void clearTimeLineContents();
 
+    void requestTimelinePage();
+
+    void updateCurrentPageNumber();
+
+    enum TimelineTypes
+    {
+        TimelineTypeMain,
+        TimelineTypeActivity,
+        TimelineTypeFavorites,
+        TimelineTypeDirect
+    };
+
+
+
 signals:
     void getPreviousPage();
     void getNextPage();
@@ -57,7 +72,6 @@ signals:
 public slots:
     void setTimeLineContents(QVariantList postList, int postsPerPage,
                              QString previousLink, QString nextLink);
-    void syncTimelineOffset(int offset);
 
     void setLikesInPost(QVariantList likesList, QString originatingPostURL);
     void setCommentsInPost(QVariantList commentsList, QString originatingPostURL);
@@ -74,12 +88,15 @@ private:
     PumpController *pController;
 
     QPushButton *firstPageButton;
+    QLabel *currentPageLabel;
     QPushButton *previousPageButton;
     QPushButton *nextPageButton;
 
     QString previousPageLink;
     QString nextPageLink;
 
+    int timelineType;
+
     int timelineOffset;
     int postsPerPage;
 
